@@ -22,10 +22,6 @@ for (my $row = $y; $row < $y + $ht; $row++) {
       $oldsecs = $newsecs;
       g_update();
    }
-
-   # also allow keyboard interaction
-   g_dokey( g_getkey() );
-
    for (my $col = $x; $col < $x + $wd; $col++) {
       g_setcell($col, $row, $maxstate - g_getcell($col, $row));
    }
