@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -534,7 +534,7 @@ hlifealgo::hlifealgo() {
    totalthings = 0 ;
    nodeblocks = 0 ;
    zeronodea = 0 ;
-   ruletable = global_liferules.rule0 ;
+   ruletable = hliferules.rule0 ;
 /*
  *   We initialize our universe to be a 16-square.  We are in drawing
  *   mode at this point.
@@ -555,7 +555,6 @@ hlifealgo::hlifealgo() {
    cacheinvalid = 0 ;
    gccount = 0 ;
    gcstep = 0 ;
-   serial = 0 ;
 }
 /**
  *   Destructor frees memory.
@@ -1525,12 +1524,12 @@ default:       return "Illegal character in readmacrocell." ;
             while (*pp > ' ') pp++ ;
             *pp = 0 ;
             
-            // AKT: need to check for B0-not-S8 rule
-            err = global_liferules.setrule(p, this);
+            // AKT: need to check for B0-not-Smax rule
+            err = hliferules.setrule(p, this);
             if (err)
                return err;
-            if (global_liferules.hasB0notS8)
-               return "B0-not-S8 rules are not allowed in HashLife.";
+            if (hliferules.alternate_rules)
+               return "B0-not-Smax rules are not allowed in HashLife.";
             
             break ;
          case 'G':
@@ -1656,20 +1655,18 @@ default:       return "Illegal character in readmacrocell." ;
 }
 const char *hlifealgo::setrule(const char *s) {
    poller->bailIfCalculating() ;
-   const char* err = global_liferules.setrule(s, this);
+   const char* err = hliferules.setrule(s, this);
    if (err) return err;
-   if (serial == global_liferules.getSerial())
-      return 0 ;
 
-   serial = global_liferules.getSerial() ;
    clearcache() ;
    
-   
-   if (global_liferules.hasB0notS8)
-      return "B0-not-S8 rules are not allowed in HashLife.";
+   if (hliferules.alternate_rules)
+      return "B0-not-Smax rules are not allowed in HashLife.";
       
-   if (global_liferules.isHexagonal())
+   if (hliferules.isHexagonal())
       grid_type = HEX_GRID;
+   else if (hliferules.isVonNeumann())
+      grid_type = VN_GRID;
    else
       grid_type = SQUARE_GRID;
       
@@ -1689,7 +1686,7 @@ void hlifealgo::unpack8x8(unsigned short nw, unsigned short ne,
  *   Write out the native macrocell format.  This is the one we use when
  *   we're not interactive and displaying a progress dialog.
  */
-g_uintptr_t hlifealgo::writecell(FILE *f, node *root, int depth) {
+g_uintptr_t hlifealgo::writecell(std::ostream &os, node *root, int depth) {
    g_uintptr_t thiscell = 0 ;
    if (root == zeronode(depth))
       return 0 ;
@@ -1715,20 +1712,20 @@ g_uintptr_t hlifealgo::writecell(FILE *f, node *root, int depth) {
          bot = (bot << 8) ;
          for (i=0; bits && i<8; i++, bits = (bits << 1) & 255)
             if (bits & 128)
-               fputs("*", f) ;
+               os << '*' ;
             else
-               fputs(".", f) ;
-         fputs("$", f) ;
+               os << '.' ;
+         os << '$' ;
       }
-      fputs("\n", f) ;
+      os << '\n' ;
    } else {
-      g_uintptr_t nw = writecell(f, root->nw, depth-1) ;
-      g_uintptr_t ne = writecell(f, root->ne, depth-1) ;
-      g_uintptr_t sw = writecell(f, root->sw, depth-1) ;
-      g_uintptr_t se = writecell(f, root->se, depth-1) ;
+      g_uintptr_t nw = writecell(os, root->nw, depth-1) ;
+      g_uintptr_t ne = writecell(os, root->ne, depth-1) ;
+      g_uintptr_t sw = writecell(os, root->sw, depth-1) ;
+      g_uintptr_t se = writecell(os, root->se, depth-1) ;
       thiscell = ++cellcounter ;
       root->next = (node *)thiscell ;
-      fprintf(f, "%d %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %" PRIuPTR "\n", depth+1, nw, ne, sw, se) ;
+      os << depth+1 << ' ' << nw << ' ' << ne << ' ' << sw << ' ' << se << '\n';
    }
    return thiscell ;
 }
@@ -1774,7 +1771,7 @@ g_uintptr_t hlifealgo::writecell_2p1(node *root, int depth) {
  *   numbered, and displaying a progress dialog.
  */
 static char progressmsg[80] ;
-g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
+g_uintptr_t hlifealgo::writecell_2p2(std::ostream &os, node *root, int depth) {
    g_uintptr_t thiscell = 0 ;
    if (root == zeronode(depth))
       return 0 ;
@@ -1783,7 +1780,7 @@ g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
          return (g_uintptr_t)(root->nw) ;
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
-         unsigned long siz = ftell(f) ;
+         std::streampos siz = os.tellp();
          sprintf(progressmsg, "File size: %.2f MB", double(siz) / 1048576.0) ;
          lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
@@ -1798,19 +1795,19 @@ g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
          bot = (bot << 8) ;
          for (i=0; bits && i<8; i++, bits = (bits << 1) & 255)
             if (bits & 128)
-               fputs("*", f) ;
+               os << '*' ;
             else
-               fputs(".", f) ;
-         fputs("$", f) ;
+               os << '.' ;
+         os << '$' ;
       }
-      fputs("\n", f) ;
+      os << '\n' ;
    } else {
       if (cellcounter + 1 > (g_uintptr_t)(root->next) || isaborted())
          return (g_uintptr_t)(root->next) ;
-      g_uintptr_t nw = writecell_2p2(f, root->nw, depth-1) ;
-      g_uintptr_t ne = writecell_2p2(f, root->ne, depth-1) ;
-      g_uintptr_t sw = writecell_2p2(f, root->sw, depth-1) ;
-      g_uintptr_t se = writecell_2p2(f, root->se, depth-1) ;
+      g_uintptr_t nw = writecell_2p2(os, root->nw, depth-1) ;
+      g_uintptr_t ne = writecell_2p2(os, root->ne, depth-1) ;
+      g_uintptr_t sw = writecell_2p2(os, root->sw, depth-1) ;
+      g_uintptr_t se = writecell_2p2(os, root->se, depth-1) ;
       if (!isaborted() &&
           cellcounter + 1 != (g_uintptr_t)(root->next)) { // this should never happen
          lifefatal("Internal in writecell_2p2") ;
@@ -1818,32 +1815,31 @@ g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
       }
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
-         unsigned long siz = ftell(f) ;
+         std::streampos siz = os.tellp();
          sprintf(progressmsg, "File size: %.2f MB", double(siz) / 1048576.0) ;
          lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
       root->next = (node *)thiscell ;
-      fprintf(f, "%d %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %" PRIuPTR "\n", depth+1, nw, ne, sw, se) ;
+      os << depth+1 << ' ' << nw << ' ' << ne << ' ' << sw << ' ' << se << '\n';
    }
    return thiscell ;
 }
 #define STRINGIFY(arg) STR2(arg)
 #define STR2(arg) #arg
-const char *hlifealgo::writeNativeFormat(FILE *f, char *comments) {
+const char *hlifealgo::writeNativeFormat(std::ostream &os, char *comments) {
    int depth = node_depth(root) ;
-   fputs("[M2] (golly " STRINGIFY(VERSION) ")", f) ;
-   fputs("\n", f) ;
+   os << "[M2] (golly " STRINGIFY(VERSION) ")\n" ;
 
    // AKT: always write out explicit rule
-   fprintf(f, "#R %s\n", global_liferules.getrule()) ;
-   
+   os << "#R " << hliferules.getrule() << '\n' ;
+
    if (generation > bigint::zero) {
       // write non-zero gen count
-      fprintf(f, "#G %s\n", generation.tostring('\0')) ;
+      os << "#G " << generation.tostring('\0') << '\n' ;
    }
    if (comments && comments[0]) {
       // write given comment line(s)
-      fputs(comments, f) ;
+      os << comments;
    }
    inGC = 1 ;
    /* this is the old way:
@@ -1870,16 +1866,17 @@ const char *hlifealgo::writeNativeFormat(FILE *f, char *comments) {
    writecells = cellcounter ;
    cellcounter = 0 ;
    if (framestosave) {
-     fprintf(f, "#FRAMES %d ", timeline.framecount) ;
-     fprintf(f, "%s ", timeline.start.tostring()) ;
-     fprintf(f, "%d^%d\n", timeline.base, timeline.expo) ;
+      os << "#FRAMES"
+         << ' ' << timeline.framecount
+         << ' ' << timeline.start.tostring()
+         << ' ' << timeline.base << '^' << timeline.expo << '\n' ;
      for (int i=0; i<timeline.framecount; i++) {
        node *frame = (node*)timeline.frames[i] ;
-       writecell_2p2(f, frame, depths[i]) ;
-       fprintf(f, "#FRAME %d %" PRIuPTR "\n", i, (g_uintptr_t)(frame->next)) ;
+       writecell_2p2(os, frame, depths[i]) ;
+       os << "#FRAME " << i << ' ' << (g_uintptr_t)frame->next ;
      }
    }
-   writecell_2p2(f, root, depth) ;
+   writecell_2p2(os, root, depth) ;
    /* end new two-pass way */
    if (framestosave) {
      for (int i=0; i<timeline.framecount; i++) {
@@ -1897,7 +1894,7 @@ void hlifealgo::doInitializeAlgoInfo(staticAlgoInfo &ai) {
    ai.setAlgorithmName("HashLife") ;
    ai.setAlgorithmCreator(&creator) ;
    ai.setDefaultBaseStep(8) ;
-   ai.setDefaultMaxMem(300) ;
+   ai.setDefaultMaxMem(500) ; // MB
    ai.minstates = 2 ;
    ai.maxstates = 2 ;
    // init default color scheme
