@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -205,7 +205,7 @@ void qlifealgo::uproot() {
      }
    }
    // Need to clear this because we don't have valid population values
-   // in the new root.  This bug and its fix was found by AKT.
+   // in the new root.
    popValid = 0 ;
 }
 /*
@@ -231,7 +231,6 @@ void qlifealgo::clearall() {
       free(memused) ;
       memused = nu ;
    }
-   serial = 0 ;
    generation = 0 ;
    increment = 1 ;
    tilelist = 0 ;
@@ -250,8 +249,6 @@ void qlifealgo::clearall() {
    emptybrick = newbrick() ;
    nullroots[0] = nullroot = root = (supertile *)(emptytile = newtile()) ;
    uproot() ;
-   // AKT: best to set ruletable after setrule call
-   // ruletable = global_liferules.rule0 ;
    popValid = 0 ;
    llxb = 0 ;
    llyb = 0 ;
@@ -1130,14 +1127,14 @@ void qlifealgo::step() {
    poller->bailIfCalculating() ;
    bigint t = increment ;
    while (t != 0) {
-      // AKT: emulate B0-not-S8 rule by changing rule table depending on gen parity
-      if (global_liferules.hasB0notS8) {
+      if (qliferules.alternate_rules) {
+         // emulate B0-not-Smax rule by changing rule table depending on gen parity
          if (generation.odd())
-            ruletable = global_liferules.rule1 ;
+            ruletable = qliferules.rule1 ;
          else
-            ruletable = global_liferules.rule0 ;
+            ruletable = qliferules.rule0 ;
       } else {
-         ruletable = global_liferules.rule0 ;
+         ruletable = qliferules.rule0 ;
       }
       dogen() ;
       if (poller->isInterrupted())
@@ -1147,48 +1144,54 @@ void qlifealgo::step() {
          t = increment ;
    }
 }
+
+// Flip bits in given rule table.
+// This is a tad tricky because we want to turn both the input
+// and the output of this table upside down.
+static void fliprule(char *rptr) {
+   for (int i=0; i<65536; i++) {
+      int j = ((i & 0xf) << 12) +
+               ((i & 0xf0) << 4) + ((i & 0xf00) >> 4) + ((i & 0xf000) >> 12) ;
+      if (i <= j) {
+         char fi = rptr[i] ;
+         char fj = rptr[j] ;
+         fi = ((fi & 0x30) >> 4) + ((fi & 0x3) << 4) ;
+         fj = ((fj & 0x30) >> 4) + ((fj & 0x3) << 4) ;
+         rptr[i] = fj ;
+         rptr[j] = fi ;
+      }
+   }
+}
+
 /**
  *   If we change the rule we need to mark everything dirty.
  */
 const char *qlifealgo::setrule(const char *s) {
-   
-   const char* err = global_liferules.setrule(s, this);
+   const char* err = qliferules.setrule(s, this);
    if (err) return err;
-   if (serial == global_liferules.getSerial())
-      return 0 ;
-   serial = global_liferules.getSerial() ;
-   markglobalchange() ;
 
-   if (!global_liferules.hasB0notS8) {
-      // current rule doesn't have B0, or it has both B0 and S8
-      ruletable = global_liferules.rule0 ;
-      if (!global_liferules.vertically_symmetrical() &&
-          !global_liferules.already_flipped()) {
-         // qlifealgo has an opposite interpretation of the orientation
-         // of ruletable than hlifealgo.  For vertically symmetrical
-         // rules, this doesn't matter.  This is a tad tricky because
-         // we want to turn both the input and the output of this function
-         // upside down.
-         for (int i=0; i<65536; i++) {
-            int j = ((i & 0xf) << 12) +
-                     ((i & 0xf0) << 4) + ((i & 0xf00) >> 4) + ((i & 0xf000) >> 12) ;
-            if (i <= j) {
-               char fi = ruletable[i] ;
-               char fj = ruletable[j] ;
-               fi = ((fi & 0x30) >> 4) + ((fi & 0x3) << 4) ;
-               fj = ((fj & 0x30) >> 4) + ((fj & 0x3) << 4) ;
-               ruletable[i] = fj ;
-               ruletable[j] = fi ;
-            }
-         }
-         global_liferules.set_flipped() ;
+   markglobalchange() ;
+   
+   // AKT: qlifealgo has an opposite interpretation of the orientation of
+   // a rule table assumed by qliferules.setrule.  For vertically symmetrical
+   // rules such as the Moore or von Neumann neighborhoods this doesn't matter,
+   // but for hexagonal rules and Wolfram rules we need to flip the rule table(s)
+   // upside down.
+   if ( qliferules.isHexagonal() || qliferules.isWolfram() ) {
+      if (qliferules.alternate_rules) {
+         // hex rule has B0 but not S6 so we'll be using rule1 for odd gens
+         fliprule(qliferules.rule1);
       }
-   } else {
-      // ruletable will be set in step() depending on gen parity
+      fliprule(qliferules.rule0);
    }
    
-   if (global_liferules.isHexagonal())
+   // ruletable is set in step(), but play safe
+   ruletable = qliferules.rule0 ;
+   
+   if (qliferules.isHexagonal())
       grid_type = HEX_GRID;
+   else if (qliferules.isVonNeumann())
+      grid_type = VN_GRID;
    else
       grid_type = SQUARE_GRID;
 
