@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -55,21 +55,28 @@ char filebuff[BUFFSIZE];
 int buffpos, bytesread, prevchar;
 
 long filesize;             // length of file in bytes
-double maxbuffs;           // length of file in buffers
-unsigned int buffcount;    // buffers read so far (for showing in progress dialog)
 
 // use buffered getchar instead of slow fgetc
 // don't override the "getchar" name which is likely to be a macro
 int mgetchar() {
    if (buffpos == BUFFSIZE) {
-#ifdef ZLIB
-      bytesread = gzread(zinstream, filebuff, BUFFSIZE);
-#else
-      bytesread = fread(filebuff, 1, BUFFSIZE, pattfile);
-#endif
+      double filepos;
+      #ifdef ZLIB
+         bytesread = gzread(zinstream, filebuff, BUFFSIZE);
+         #if ZLIB_VERNUM >= 0x1240
+            // gzoffset is only available in zlib 1.2.4 or later
+            filepos = gzoffset(zinstream);
+         #else
+            // use an approximation of file position if file is compressed
+            filepos = gztell(zinstream);
+            if (filepos > 0 && gzdirect(zinstream) == 0) filepos /= 4;
+         #endif
+      #else
+         bytesread = fread(filebuff, 1, BUFFSIZE, pattfile);
+         filepos = ftell(filebuff);
+      #endif
       buffpos = 0;
-      buffcount++;
-      lifeabortprogress((double)buffcount / maxbuffs, "");
+      lifeabortprogress(filepos / filesize, "");
    }
    if (buffpos >= bytesread) return EOF;
    return filebuff[buffpos++];
@@ -548,7 +555,6 @@ const char *readmcell(lifealgo &imp, char *line) {
 }
 
 long getfilesize(const char *filename) {
-   // following method is only accurate for uncompressed file
    long flen = 0;
    FILE *f = fopen(filename, "r");
    if (f != 0) {
@@ -556,15 +562,47 @@ long getfilesize(const char *filename) {
       flen = ftell(f);
       fclose(f);
    }
-   const char *p = strrchr(filename, '.');
-   if ( p != NULL && p[1] == 'g' && p[2] == 'z' ) {
-      // is there a fast way to get uncompressed size of a .gz file???!!!
-      // for now we just multiply flen by a fudge factor -- yuk!
-      flen = long( (double)flen * 4.0 );
-   }
    return flen;
 }
 
+// This function guesses whether `line' is the start of a headerless Life RLE
+// pattern.  It is used to distinguish headerless RLE from plain text patterns.
+static bool isplainrle(const char *line) {
+
+   // Find end of line, or terminating '!' character, whichever comes first:
+   const char *end = line;
+   while (*end && *end != '!') ++end;
+
+   // Verify that '!' (if present) is the final printable character:
+   if (*end == '!') {
+      for (const char *p = end + 1; *p; ++p) {
+         if ((unsigned)*p > ' ') {
+            return false;
+         }
+      }
+   }
+
+   // Ensure line consists of valid tokens:
+   bool prev_digit = false, have_digit = false;
+   for (const char *p = line; p != end; ++p) {
+      if ((unsigned)*p <= ' ') {
+         if (prev_digit) return false;  // space inside token!
+      } else if (*p >= '0' && *p <= '9') {
+         prev_digit = have_digit = true;
+      } else if (*p == 'b' || *p == 'o' || *p == '$') {
+         prev_digit = false;
+      } else {
+         return false;  // unsupported printable character encountered!
+      }
+   }
+   if (prev_digit) return false;  // end of line inside token!
+
+   // Everything seems parseable; assume this is RLE if either we saw some
+   // digits, or the pattern ends with a '!', both of which are unlikely to
+   // occur in plain text patterns:
+   return have_digit || *end == '!';
+}
+
 const char *loadpattern(lifealgo &imp) {
    char line[LINESIZE + 1] ;
    const char *errmsg = 0;
@@ -585,9 +623,6 @@ const char *loadpattern(lifealgo &imp) {
       if (err) imp.setrule( imp.DefaultRule() ) ;
    }
 
-   buffcount = 0;
-   maxbuffs = (double)filesize / (double)BUFFSIZE;
-   if (maxbuffs < 1.0) maxbuffs = 1.0;
    if (getedges)
       lifebeginprogress("Reading from clipboard");
    else
@@ -625,7 +660,18 @@ const char *loadpattern(lifealgo &imp) {
    } else if (line[0] == '#' || line[0] == 'x') {
       errmsg = readrle(imp, line) ;
       imp.endofpattern() ;
-      // if getedges is true then readrle has set top,left,bottom,right
+      if (getedges && !imp.isEmpty()) {
+         // readrle has set top,left,bottom,right based on the info given in
+         // the header line and possibly a "#CXRLE Pos=..." line, but in case
+         // that info is incorrect we find the true pattern edges and expand
+         // top/left/bottom/right if necessary to avoid truncating the pattern
+         bigint t, l, b, r ;
+         imp.findedges(&t, &l, &b, &r) ;
+         if (t < top) top = t ;
+         if (l < left) left = l ;
+         if (b > bottom) bottom = b ;
+         if (r > right) right = r ;
+      }
 
    } else if (line[0] == '!') {
       errmsg = readdblife(imp, line) ;
@@ -641,6 +687,13 @@ const char *loadpattern(lifealgo &imp) {
          imp.findedges(&top, &left, &bottom, &right) ;
       }
 
+   } else if (isplainrle(line)) {
+      errmsg = readrle(imp, line) ;
+      imp.endofpattern() ;
+      if (getedges && !imp.isEmpty()) {
+         imp.findedges(&top, &left, &bottom, &right) ;
+      }
+
    } else {
       // read a text pattern like "...ooo$$$ooo"
       errmsg = readtextpattern(imp, line) ;
@@ -747,9 +800,6 @@ const char *readcomments(const char *filename, char **commptr)
    // loading comments is likely to be quite fast so no real need to
    // display the progress dialog, but getchar calls lifeabortprogress
    // so safer just to assume the progress dialog might appear
-   buffcount = 0;
-   maxbuffs = (double)filesize / (double)BUFFSIZE;
-   if (maxbuffs < 1.0) maxbuffs = 1.0;
    lifebeginprogress("Loading comments");
 
    // skip any blank lines at start to avoid problem when copying pattern
