@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -53,7 +53,7 @@ protected:
                         
 protected:
 
-   string current_rule;
+   std::string current_rule;
    unsigned int n_states;
    enum TNeighborhood { vonNeumann, Moore, hexagonal, oneDimensional } neighborhood; 
    static const int N_SUPPORTED_NEIGHBORHOODS = 4;
