@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -72,11 +72,15 @@ public:
 
 extern AlgoData* algoinfo[MAX_ALGOS];     // static info for each algorithm
 extern wxMenu* algomenu;                  // menu of algorithm names
+extern wxMenu* algomenupop;               // copy of algomenu for PopupMenu calls
 extern algo_type initalgo;                // initial algorithm
 
 extern wxBitmap** hexicons7x7;          // hexagonal icon bitmaps for scale 1:8
 extern wxBitmap** hexicons15x15;        // hexagonal icon bitmaps for scale 1:16
 
+extern wxBitmap** vnicons7x7;          // diamond icon bitmaps for scale 1:8
+extern wxBitmap** vnicons15x15;        // diamond icon bitmaps for scale 1:16
+
 void InitAlgorithms();
 // Initialize above data.  Must be called before reading the prefs file.
 
