@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -97,7 +97,8 @@ bool MainFrame::SaveStartingPattern()
    // use CanWriteFormat(MC_format)???
    if ( currlayer->algo->hyperCapable() ) {
       // much faster to save pattern in a macrocell file
-      const char* err = WritePattern(currlayer->tempstart, MC_format, 0, 0, 0, 0);
+      const char* err = WritePattern(currlayer->tempstart, MC_format,
+                                     no_compression, 0, 0, 0, 0);
       if (err) {
          statusptr->ErrorMessage(wxString(err,wxConvLocal));
          // don't allow user to continue generating
@@ -119,6 +120,7 @@ bool MainFrame::SaveStartingPattern()
       // use XRLE format so the pattern's top left location and the current
       // generation count are stored in the file
       const char* err = WritePattern(currlayer->tempstart, XRLE_format,
+                                     no_compression,
                                      itop, ileft, ibottom, iright);
       if (err) {
          statusptr->ErrorMessage(wxString(err,wxConvLocal));
@@ -199,8 +201,8 @@ void MainFrame::ResetPattern(bool resetundo)
    currlayer->savestart = !currlayer->startfile.IsEmpty();
    
    // restore settings saved by SaveStartingPattern
+   RestoreRule(currlayer->startrule);
    currlayer->currname = currlayer->startname;
-   currlayer->algo->setrule(currlayer->startrule.mb_str(wxConvLocal));
    currlayer->dirty = currlayer->startdirty;
    if (restoreview) {
       viewptr->SetPosMag(currlayer->startx, currlayer->starty, currlayer->startmag);
@@ -225,7 +227,7 @@ void MainFrame::ResetPattern(bool resetundo)
             cloneptr->currexpo = cloneptr->startexpo;
             // also synchronize dirty flags and update items in Layer menu
             cloneptr->dirty = currlayer->dirty;
-            mainptr->UpdateLayerItem(i);
+            UpdateLayerItem(i);
          }
       }
    }
@@ -333,7 +335,11 @@ const char* MainFrame::ChangeGenCount(const char* genstring, bool inundoredo)
       }
       // create a new universe of same type and same rule
       lifealgo* newalgo = CreateNewUniverse(currlayer->algtype);
-      newalgo->setrule(currlayer->algo->getrule());
+      const char* err = newalgo->setrule(currlayer->algo->getrule());
+      if (err) {
+         delete newalgo;
+         return "Current rule is no longer valid!";
+      }
       newalgo->setGeneration(newgen);
       // copy pattern
       if ( !viewptr->CopyRect(top.toint(), left.toint(), bottom.toint(), right.toint(),
@@ -468,12 +474,11 @@ void MainFrame::DisplayPattern()
    if (!IsIconized()) {
       if (tilelayers && numlayers > 1 && !syncviews && currlayer->cloneid == 0) {
          // only update the current tile
-         #if defined(__WXMAC__)
-            // Refresh + Update is too slow on Mac
-            wxClientDC dc(viewptr);
-            DrawView(dc, viewptr->tileindex);
+         viewptr->Refresh(false);
+         #ifdef __WXMAC__
+            if (!showstatus) viewptr->Update();
+            // else let statusptr->Update() update viewport
          #else
-            viewptr->Refresh(false);
             viewptr->Update();
          #endif
       } else {
@@ -481,14 +486,18 @@ void MainFrame::DisplayPattern()
          // (tile windows are children of bigview)
          if (numlayers > 1 && (stacklayers || tilelayers)) {
             bigview->Refresh(false);
-            bigview->Update();
+            #ifdef __WXMAC__
+               if (!showstatus) bigview->Update();
+               // else let statusptr->Update() update viewport
+            #else
+               bigview->Update();
+            #endif
          } else {
-            #if defined(__WXMAC__)
-               // Refresh + Update is too slow on Mac
-               wxClientDC dc(viewptr);
-               DrawView(dc, viewptr->tileindex);
+            viewptr->Refresh(false);
+            #ifdef __WXMAC__
+               if (!showstatus) viewptr->Update();
+               // else let statusptr->Update() update viewport
             #else
-               viewptr->Refresh(false);
                viewptr->Update();
             #endif
          }
@@ -1279,8 +1288,23 @@ void MainFrame::GeneratePattern()
    // to test inscript or currlayer->stayclean; note that we must call
    // RememberGenFinish BEFORE processing any pending command
    if (allowundo) currlayer->undoredo->RememberGenFinish();
+
+   // stop recording any timeline before processing any pending command
+   if (currlayer->algo->isrecording()) {
+      currlayer->algo->stoprecording();
+      if (currlayer->algo->getframecount() > 0) {
+         // probably best to go to last frame
+         currlayer->currframe = currlayer->algo->getframecount() - 1;
+         currlayer->autoplay = 0;
+         currlayer->tlspeed = 0;
+         currlayer->algo->gotoframe(currlayer->currframe);
+         if (currlayer->autofit) viewptr->FitInView(1);
+      }
+      if (!showtimeline) ToggleTimelineBar();
+      UpdateUserInterface(true);
+   }
    
-   DoPendingAction(true);     // true means can restart generating loop
+   DoPendingAction(true);     // true means we can restart generating loop
 }
 
 // -----------------------------------------------------------------------------
@@ -1298,7 +1322,7 @@ void MainFrame::DoPendingAction(bool restart)
          case ID_OPEN_CLIP:      OpenClipboard(); break;
          case ID_RESET:          ResetPattern(); break;
          case ID_SETGEN:         SetGeneration(); break;
-         case wxID_UNDO:         currlayer->undoredo->UndoChange(); break;
+         case ID_UNDO:           currlayer->undoredo->UndoChange(); break;
          case ID_ADD_LAYER:      AddLayer(); break;
          case ID_DUPLICATE:      DuplicateLayer(); break;
          case ID_LOAD_LEXICON:   LoadLexiconPattern(); break;
@@ -1362,8 +1386,8 @@ void MainFrame::DoPendingAction(bool restart)
                
                // process the pending command
                cmdevent.SetEventType(wxEVT_COMMAND_MENU_SELECTED);
-               cmdevent.SetEventObject(mainptr);
-               mainptr->ProcessEvent(cmdevent);
+               cmdevent.SetEventObject(this);
+               this->GetEventHandler()->ProcessEvent(cmdevent);
                
                // restore tool/layer/edit bar flags
                showtool = saveshowtool;
@@ -1393,7 +1417,7 @@ void MainFrame::DoPendingAction(bool restart)
       // do the drawing
       mouseevent.SetEventType(wxEVT_LEFT_DOWN);
       mouseevent.SetEventObject(viewptr);
-      viewptr->ProcessEvent(mouseevent);
+      viewptr->GetEventHandler()->ProcessEvent(mouseevent);
       while (viewptr->drawingcells) {
          wxGetApp().Yield(true);
          wxMilliSleep(5);             // don't hog CPU
@@ -1425,7 +1449,7 @@ void MainFrame::DisplayTimingInfo()
       double secs = (double)(endtime - begintime) / 1000.0;
       double gens = endgen - begingen;
       wxString s;
-      s.Printf(_("%g gens in %g secs (%g gens/sec)."), gens, secs, gens / secs);
+      s.Printf(_("%g gens in %g secs (%g gens/sec)."), gens, secs, gens/secs);
       statusptr->DisplayMessage(s);
    }
 }
@@ -1516,6 +1540,11 @@ void MainFrame::NextGeneration(bool useinc)
    // avoid doing some things if NextGeneration is called from a script;
    // ie. by a run/step command
    if (!inscript) {
+      if (useinc) {
+         // for DisplayTimingInfo
+         begintime = stopwatch->Time();
+         begingen = curralgo->getGeneration().todouble();
+      }
       wxGetApp().PollerReset();
       viewptr->CheckCursor(IsActive());
    }
@@ -1567,6 +1596,11 @@ void MainFrame::NextGeneration(bool useinc)
    lifealgo::setVerbose(0);
    
    if (!inscript) {
+      if (useinc) {
+         // for DisplayTimingInfo (we add 1 millisec here in case it took < 1 millisec)
+         endtime = stopwatch->Time() + 1;
+         endgen = curralgo->getGeneration().todouble();
+      }
       // autofit is only used when doing many gens
       if (currlayer->autofit && useinc && curralgo->getIncrement() > bigint::one)
          viewptr->FitInView(0);
@@ -1811,9 +1845,11 @@ void MainFrame::ShowRuleDialog()
       // the user might have simply added/deleted a named rule
       SetWindowTitle(wxEmptyString);
       
-      // check if rule actually changed
+      // check if the rule string changed, or the number of states changed
+      // (the latter might happen if user edited a table/tree file)
       wxString newrule = wxString(currlayer->algo->getrule(), wxConvLocal);
-      if (oldrule != newrule) {
+      int newmaxstate = currlayer->algo->NumCellStates() - 1;
+      if (oldrule != newrule || oldmaxstate != newmaxstate) {
          // if grid is bounded then remove any live cells outside grid edges
          if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
             ClearOutsideGrid();
@@ -1821,18 +1857,21 @@ void MainFrame::ShowRuleDialog()
          
          // rule change might have changed the number of cell states;
          // if there are fewer states then pattern might change
-         int newmaxstate = currlayer->algo->NumCellStates() - 1;
          if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
             ReduceCellStates(newmaxstate);
          }
          
-         // pattern might have changed or new rule might have changed colors
-         UpdateEverything();
-         
          if (allowundo) {
             currlayer->undoredo->RememberRuleChange(oldrule);
          }
       }
+
+      // switch to default colors and icons for new rule (we need to do this even if
+      // oldrule == newrule in case there's a new/changed .colors or .icons file)
+      UpdateLayerColors();
+      
+      // pattern or colors or icons might have changed
+      UpdateEverything();
    }
 }
 
@@ -1887,9 +1926,9 @@ void MainFrame::ChangeAlgorithm(algo_type newalgotype, const wxString& newrule,
    lifealgo* newalgo = CreateNewUniverse(newalgotype);
    
    if (inundoredo) {
-      // switch to given newrule (no error should occur)
+      // switch to given newrule
       const char* err = newalgo->setrule( newrule.mb_str(wxConvLocal) );
-      if (err) Warning(_("Bug detected in ChangeAlgorithm!"));
+      if (err) newalgo->setrule( newalgo->DefaultRule() );
    } else {
       const char* err;
       if (newrule.IsEmpty()) {
