@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -88,6 +88,15 @@ enum {
    #include "bitmaps/allstates_down.xpm"
 #endif
 
+// width and height of bitmap buttons
+#if defined(__WXOSX_COCOA__) || defined(__WXGTK__)
+   const int BUTTON_WD = 28;
+   const int BUTTON_HT = 28;
+#else
+   const int BUTTON_WD = 24;
+   const int BUTTON_HT = 24;
+#endif
+
 // -----------------------------------------------------------------------------
 
 // Define edit bar window:
@@ -253,7 +262,7 @@ EditBar::EditBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int ht)
       smallgap = 6;
    #else
       xpos = 4;
-      ypos = 4;
+      ypos = (32 - BUTTON_HT) / 2;
       smallgap = 4;
    #endif
    biggap = 16;
@@ -287,6 +296,10 @@ EditBar::EditBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int ht)
       // use smaller font on GTK
       editfont = wxFont::New(8, wxMODERN, wxNORMAL, wxNORMAL);
       textascent = 11;
+   #elif defined(__WXOSX_COCOA__)
+      // we need to specify facename to get Monaco instead of Courier
+      editfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Monaco"));
+      textascent = 10;
    #elif defined(__WXMAC__)
       editfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL);
       textascent = 10;
@@ -627,8 +640,8 @@ void EditBar::OnButton(wxCommandEvent& event)
 
    int cmdid;
    switch (id) {
-      case UNDO_BUTT:      cmdid = wxID_UNDO; break;
-      case REDO_BUTT:      cmdid = wxID_REDO; break;
+      case UNDO_BUTT:      cmdid = ID_UNDO; break;
+      case REDO_BUTT:      cmdid = ID_REDO; break;
       case DRAW_BUTT:      cmdid = ID_DRAW; break;
       case PICK_BUTT:      cmdid = ID_PICK; break;
       case SELECT_BUTT:    cmdid = ID_SELECT; break;
@@ -642,6 +655,9 @@ void EditBar::OnButton(wxCommandEvent& event)
    // call MainFrame::OnMenu after OnButton finishes
    wxCommandEvent cmdevt(wxEVT_COMMAND_MENU_SELECTED, cmdid);
    wxPostEvent(mainptr->GetEventHandler(), cmdevt);
+
+   // avoid possible problems
+   viewptr->SetFocus();
 }
 
 // -----------------------------------------------------------------------------
@@ -733,7 +749,7 @@ void EditBar::OnButtonUp(wxMouseEvent& event)
       // call OnButton
       wxCommandEvent buttevt(wxEVT_COMMAND_BUTTON_CLICKED, id);
       buttevt.SetEventObject(ebbutt[id]);
-      ebbutt[id]->ProcessEvent(buttevt);
+      ebbutt[id]->GetEventHandler()->ProcessEvent(buttevt);
    }
 }
 
@@ -741,11 +757,11 @@ void EditBar::OnButtonUp(wxMouseEvent& event)
 
 void EditBar::AddButton(int id, const wxString& tip)
 {
-   ebbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos));
+   ebbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos),
+                                   wxSize(BUTTON_WD, BUTTON_HT));
    if (ebbutt[id] == NULL) {
       Fatal(_("Failed to create edit bar button!"));
    } else {
-      const int BUTTON_WD = 24;        // nominal width of bitmap buttons
       xpos += BUTTON_WD + smallgap;
       ebbutt[id]->SetToolTip(tip);
       #ifdef __WXMSW__
