@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -58,7 +58,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wxhelp.h"        // for ShowHelp
 #include "wxtimeline.h"    // for InitTimelineFrame, ToggleTimelineBar, etc
 
-#ifdef __WXMAC__
+#if defined(__WXMAC__) && !defined(__WXOSX_COCOA__)
    #include <Carbon/Carbon.h>                      // for OpaqueWindowPtr, etc
    #include "wx/mac/corefoundation/cfstring.h"     // for wxMacCFStringHolder
 #endif
@@ -70,6 +70,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
    #define FILEPATH path.mb_str(wxConvLocal)
 #endif
 
+#if wxCHECK_VERSION(2,9,0)
+// some wxMenuItem method names have changed in wx 2.9
+   #define GetText GetItemLabel
+   #define SetText SetItemLabel
+#endif
+
 // File menu functions:
 
 // -----------------------------------------------------------------------------
@@ -84,7 +90,8 @@ wxString MainFrame::GetBaseName(const wxString& path)
 
 void MainFrame::MySetTitle(const wxString& title)
 {
-   #ifdef __WXMAC__
+   // fix if SetTitle still causes window refresh in Cocoa version!!!
+   #if defined(__WXMAC__) && !defined(__WXOSX_COCOA__)
       // avoid wxMac's SetTitle call -- it causes an undesirable window refresh
       SetWindowTitleWithCFString((OpaqueWindowPtr*)this->MacGetWindowRef(),
                                  wxMacCFStringHolder(title, wxFONTENCODING_DEFAULT));
@@ -166,7 +173,7 @@ void MainFrame::CreateUniverse()
    currlayer->algo = CreateNewUniverse(currlayer->algtype);
    
    // ensure new universe uses same rule (and thus same # of cell states)
-   currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+   RestoreRule(oldrule);
 
    // increment has been reset to 1 but that's probably not always desirable
    // so set increment using current step size
@@ -205,8 +212,6 @@ void MainFrame::NewPattern(const wxString& title)
    // clear all undo/redo history
    currlayer->undoredo->ClearUndoRedo();
 
-   // rule doesn't change so no need to call setrule
-
    if (newremovesel) currlayer->currsel.Deselect();
    if (newcurs) currlayer->curs = newcurs;
    viewptr->SetPosMag(bigint::zero, bigint::zero, newmag);
@@ -353,7 +358,7 @@ void MainFrame::LoadPattern(const wxString& path, const wxString& newtitle,
 
    if (IsImageFile(path)) {
       // ensure new universe uses same rule
-      currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+      RestoreRule(oldrule);
       LoadImage(path);
       viewptr->nopattupdate = false;
    } else {
@@ -376,7 +381,7 @@ void MainFrame::LoadPattern(const wxString& path, const wxString& newtitle,
             currlayer->algtype = oldalgo;
             delete currlayer->algo;
             currlayer->algo = CreateNewUniverse(currlayer->algtype);
-            currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+            RestoreRule(oldrule);
             // Warning( wxString(err,wxConvLocal) );
             // current error and original error are not necessarily meaningful
             // so report a more generic error
@@ -570,14 +575,14 @@ void MainFrame::OpenZipFile(const wxString& zippath)
    contents += zippath;
    contents += wxT("<p>\n");
    contents += wxT("Contents:<br>\n");
-   
+      
    wxFFileInputStream instream(zippath);
    if (!instream.Ok()) {
       Warning(_("Could not create input stream for zip file:\n") + zippath);
       return;
    }
    wxZipInputStream zip(instream);
-   
+      
    // examine each entry in zip file and build contents string;
    // also install any .table/tree/colors/icons files
    wxZipEntry* entry;
@@ -683,7 +688,7 @@ void MainFrame::OpenZipFile(const wxString& zippath)
       contents += wxT("\n");
    }
    contents += wxT("\n</body></html>");
-   
+      
    if (dirseen || rulefiles > 0 || textfiles > 0 || patternfiles > 1 || scriptfiles > 1) {
       // complex zip, so write contents to a temporary html file and display it in help window;
       // use a unique file name so user can go back/forwards
@@ -926,10 +931,10 @@ void MainFrame::OpenPattern()
    wxString filetypes = _("All files (*)|*");
    filetypes +=         _("|RLE (*.rle)|*.rle");
    filetypes +=         _("|Macrocell (*.mc)|*.mc");
+   filetypes +=         _("|Gzip (*.gz)|*.gz");
    filetypes +=         _("|Life 1.05/1.06 (*.lif)|*.lif");
    filetypes +=         _("|dblife (*.l)|*.l");
    filetypes +=         _("|MCell (*.mcl)|*.mcl");
-   filetypes +=         _("|Gzip (*.gz)|*.gz");
    filetypes +=         _("|Zip (*.zip;*.gar)|*.zip;*.gar");
    filetypes +=         _("|BMP (*.bmp)|*.bmp");
    filetypes +=         _("|GIF (*.gif)|*.gif");
@@ -1005,6 +1010,12 @@ bool MainFrame::CopyTextToClipboard(const wxString& text)
 
 // -----------------------------------------------------------------------------
 
+#if wxCHECK_VERSION(2,9,0)
+   // wxTextDataObject also has GetText and SetText methods so don't change them
+   #undef GetText
+   #undef SetText
+#endif
+
 bool MainFrame::GetTextFromClipboard(wxTextDataObject* textdata)
 {
    bool gotdata = false;
@@ -1217,6 +1228,12 @@ void MainFrame::RunClipboard()
    }
 }
 
+#if wxCHECK_VERSION(2,9,0)
+   // restore new wxMenuItem method names in wx 2.9
+   #define GetText GetItemLabel
+   #define SetText SetItemLabel
+#endif
+
 // -----------------------------------------------------------------------------
 
 void MainFrame::OpenRecentPattern(int id)
@@ -1407,6 +1424,7 @@ void MainFrame::ClearAllScripts()
 
 const char* MainFrame::WritePattern(const wxString& path,
                                     pattern_format format,
+                                    output_compression compression,
                                     int top, int left, int bottom, int right)
 {
    // if the format is RLE_format and the grid is bounded then force XRLE_format so that
@@ -1414,7 +1432,7 @@ const char* MainFrame::WritePattern(const wxString& path,
    if (format == RLE_format && (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0))
       format = XRLE_format;
    const char* err = writepattern(FILEPATH, *currlayer->algo, format,
-                                  top, left, bottom, right);
+                                  compression, top, left, bottom, right);
 
    #ifdef __WXMAC__
       if (!err) {
@@ -1424,10 +1442,13 @@ const char* MainFrame::WritePattern(const wxString& path,
          wxUint32 type = 'GoLR';       // RLE or XRLE
          if (format == MC_format) {
             type = 'GoLM';
-         } else if (format == L105_format) {
-            type = 'GoLL';
          }
-         filename.MacSetTypeAndCreator(type, creator);
+         #if defined(__WXOSX_COCOA__)
+            // there is no Cocoa call to set file type and creator
+            creator = 'GoLy';          // avoid compiler warning
+         #else
+            filename.MacSetTypeAndCreator(type, creator);
+         #endif
       }
    #endif
 
@@ -1447,38 +1468,45 @@ bool MainFrame::SavePattern()
    }
 
    wxString filetypes;
-   int RLEindex, L105index, MCindex;
+   int MCindex, RLEindex;
 
    // initially all formats are not allowed (use any -ve number)
-   RLEindex = L105index = MCindex = -1;
+   MCindex = RLEindex = -1;
+
+   // adding "*.gz" to the file types avoids a duplication bug in the wxOSX app
+   wxString MCfiles, RLEfiles;
+   MCfiles = _("Macrocell (*.mc)|*.mc");
+   MCfiles += _("|Compressed Macrocell (*.mc.gz)|*.mc.gz;*.gz");
+   if (savexrle) {
+      RLEfiles = _("Extended RLE (*.rle)|*.rle");
+      RLEfiles += _("|Compressed Extended RLE (*.rle.gz)|*.rle.gz;*.gz");
+   } else {
+      RLEfiles = _("RLE (*.rle)|*.rle");
+      RLEfiles += _("|Compressed RLE (*.rle.gz)|*.rle.gz;*.gz");
+   }
 
    bigint top, left, bottom, right;
    int itop, ileft, ibottom, iright;
    currlayer->algo->findedges(&top, &left, &bottom, &right);
 
-   wxString RLEstring;
-   if (savexrle)
-      RLEstring = _("Extended RLE (*.rle)|*.rle");
-   else
-      RLEstring = _("RLE (*.rle)|*.rle");
-
-   //!!! need currlayer->algo->CanWriteMC()???
    if (currlayer->algo->hyperCapable()) {
+      // algorithm uses hashlife
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
          // too big so only allow saving as MC file
          itop = ileft = ibottom = iright = 0;
-         filetypes = _("Macrocell (*.mc)|*.mc");
+         filetypes = MCfiles;
          MCindex = 0;
       } else {
-         // allow saving as RLE/MC file
+         // allow saving as MC or RLE file
          itop = top.toint();
          ileft = left.toint();
          ibottom = bottom.toint();
          iright = right.toint();
-         filetypes = RLEstring;
-         RLEindex = 0;
-         filetypes += _("|Macrocell (*.mc)|*.mc");
-         MCindex = 1;
+         filetypes = MCfiles;
+         filetypes += _("|");
+         filetypes += RLEfiles;
+         MCindex = 0;
+         RLEindex = 1;
       }
    } else {
       // allow saving file only if pattern is small enough
@@ -1490,12 +1518,8 @@ bool MainFrame::SavePattern()
       ileft = left.toint();
       ibottom = bottom.toint();
       iright = right.toint();
-      filetypes = RLEstring;
+      filetypes = RLEfiles;
       RLEindex = 0;
-      /* Life 1.05 format not yet implemented!!!
-      filetypes += _("|Life 1.05 (*.lif)|*.lif");
-      L105index = 1;
-      */
    }
 
    wxFileDialog savedlg( this, _("Save pattern"),
@@ -1512,28 +1536,30 @@ bool MainFrame::SavePattern()
       opensavedir = fullpath.GetPath();
       wxString ext = fullpath.GetExt();
       pattern_format format;
+      output_compression compression = no_compression;
+      // detect if user supplied a compression suffix (.gz)
+      if ( ext.IsSameAs(wxT("gz"),false) ) {
+         compression = gzip_compression;
+         ext = wxFileName(fullpath.GetName()).GetExt();
+      }
       // if user supplied a known extension then use that format if it is
       // allowed, otherwise use current format specified in filter menu
       if ( ext.IsSameAs(wxT("rle"),false) && RLEindex >= 0 ) {
          format = savexrle ? XRLE_format : RLE_format;
-      /* Life 1.05 format not yet implemented!!!
-      } else if ( ext.IsSameAs("lif",false) && L105index >= 0 ) {
-         format = L105_format;
-      */
       } else if ( ext.IsSameAs(wxT("mc"),false) && MCindex >= 0 ) {
          format = MC_format;
-      } else if ( savedlg.GetFilterIndex() == RLEindex ) {
-         format = savexrle ? XRLE_format : RLE_format;
-      } else if ( savedlg.GetFilterIndex() == L105index ) {
-         format = L105_format;
-      } else if ( savedlg.GetFilterIndex() == MCindex ) {
+      } else if ( savedlg.GetFilterIndex()/2 == MCindex ) {
          format = MC_format;
+         if (savedlg.GetFilterIndex()%2) compression = gzip_compression;
+      } else if ( savedlg.GetFilterIndex()/2 == RLEindex ) {
+         format = savexrle ? XRLE_format : RLE_format;
+         if (savedlg.GetFilterIndex()%2) compression = gzip_compression;
       } else {
          statusptr->ErrorMessage(_("Bug in SavePattern!"));
          return false;
       }
 
-      const char* err = WritePattern(savedlg.GetPath(), format,
+      const char* err = WritePattern(savedlg.GetPath(), format, compression,
                                      itop, ileft, ibottom, iright);
       if (err) {
          statusptr->ErrorMessage(wxString(err,wxConvLocal));
@@ -1550,15 +1576,21 @@ bool MainFrame::SavePattern()
 // -----------------------------------------------------------------------------
 
 // called by script command to save current pattern to given file
-const char* MainFrame::SaveFile(const wxString& path, const wxString& format, bool remember)
+const char* MainFrame::SaveFile(const wxString& path, const wxString& fileformat, bool remember)
 {
-   // check that given format is valid and allowed
    bigint top, left, bottom, right;
    int itop, ileft, ibottom, iright;
    currlayer->algo->findedges(&top, &left, &bottom, &right);
 
+   wxString format = fileformat.Lower();
+   output_compression compression = no_compression;
+   if (format.EndsWith(wxT(".gz"))) {
+      compression = gzip_compression;
+   }
+   
+   // check that given file format is valid
    pattern_format pattfmt;
-   if ( format.IsSameAs(wxT("rle"),false) ) {
+   if (format.StartsWith(wxT("rle"))) {
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
          return "Pattern is too big to save as RLE.";
       }
@@ -1567,8 +1599,7 @@ const char* MainFrame::SaveFile(const wxString& path, const wxString& format, bo
       ileft = left.toint();
       ibottom = bottom.toint();
       iright = right.toint();
-   } else if ( format.IsSameAs(wxT("mc"),false) ) {
-      //!!! need currlayer->algo->CanWriteMC()???
+   } else if (format.StartsWith(wxT("mc"))) {
       if (!currlayer->algo->hyperCapable()) {
          return "Macrocell format is not supported by the current algorithm.";
       }
@@ -1579,7 +1610,8 @@ const char* MainFrame::SaveFile(const wxString& path, const wxString& format, bo
       return "Unknown pattern format.";
    }
 
-   const char* err = WritePattern(path, pattfmt, itop, ileft, ibottom, iright);
+   const char* err = WritePattern(path, pattfmt, compression,
+                                  itop, ileft, ibottom, iright);
    if (!err) {
       if (remember) AddRecentPattern(path);
       SaveSucceeded(path);
