@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -127,6 +127,7 @@ public:
    }
    
    bool editlink;       // open clicked file in editor?
+   bool canreload;      // can OnSize call CheckAndLoad?
 
 private:
    #ifdef __WXMSW__
@@ -318,9 +319,15 @@ void ShowHelp(const wxString& filepath)
       } else {
          htmlwin->CheckAndLoad(currhelp);
       }
+         
+      // prevent HtmlView::OnSize calling CheckAndLoad twice
+      htmlwin->canreload = false;
 
       helpptr->Show(true);
       UpdateHelpButtons();    // must be after Show to avoid hbar appearing on Mac
+         
+      // allow HtmlView::OnSize to call CheckAndLoad if window is resized
+      htmlwin->canreload = true;
    }
    whenactive = 0;
 }
@@ -440,7 +447,7 @@ void LoadRule(const wxString& rulestring)
                // change the current algorithm and switch to the new rule
                mainptr->ChangeAlgorithm(i, rulestring);
                if (i != currlayer->algtype) {
-                  currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+                  RestoreRule(oldrule);
                   Warning(_("Algorithm could not be changed (pattern is too big to convert)."));
                   return;
                } else {
@@ -452,7 +459,7 @@ void LoadRule(const wxString& rulestring)
          }
       }
       // should only get here if table/tree file contains some sort of error
-      currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+      RestoreRule(oldrule);
       Warning(_("Rule is not valid in any algorithm: ") + rulestring);
       return;
    }
@@ -588,7 +595,8 @@ void GetURL(const wxString& url)
 
    wxString filename = fullurl.AfterLast('/');
    // remove ugly stuff at start of file names downloaded from ConwayLife.com
-   if (filename.StartsWith(wxT("pattern.asp?p=")) ||
+   if (filename.StartsWith(wxT("download.php?f=")) ||
+       filename.StartsWith(wxT("pattern.asp?p=")) ||
        filename.StartsWith(wxT("script.asp?s="))) {
       filename = filename.AfterFirst('=');
    }
@@ -991,6 +999,12 @@ void HtmlView::ClearStatus()
 
 // -----------------------------------------------------------------------------
 
+#if defined(__WXMAC__) && wxCHECK_VERSION(2,9,0)
+   // wxMOD_CONTROL has been changed to mean Command key down (sheesh!)
+   #define wxMOD_CONTROL wxMOD_RAW_CONTROL
+   #define ControlDown RawControlDown
+#endif
+
 void HtmlView::OnMouseDown(wxMouseEvent& event)
 {
    // set flag so ctrl/right-clicked file can be opened in editor
@@ -1155,7 +1169,7 @@ void HtmlView::OnSize(wxSizeEvent& event)
    wxHtmlWindow::OnSize(event);
 
    wxString currpage = GetOpenedPage();
-   if ( !currpage.IsEmpty() ) {
+   if ( !currpage.IsEmpty() && canreload ) {
       CheckAndLoad(currpage);    // reload page
       Scroll(x, y);              // scroll to old position
    }
@@ -1172,7 +1186,12 @@ void HtmlView::OnTimer(wxTimerEvent& WXUNUSED(event))
       // send idle event to html window so cursor gets updated
       // even while app is busy doing something else (eg. generating)
       wxIdleEvent idleevent;
-      wxGetApp().SendIdleEvents(this, idleevent);
+      #if wxCHECK_VERSION(2,9,2)
+         // SendIdleEvents is now in wxWindow
+         SendIdleEvents(idleevent);
+      #else
+         wxGetApp().SendIdleEvents(this, idleevent);
+      #endif
    }
 }
 
@@ -1189,7 +1208,11 @@ void HtmlView::SetFontSizes(int size)
    f_sizes[4] = int(size * 1.4);
    f_sizes[5] = int(size * 1.6);
    f_sizes[6] = int(size * 1.8);
-   SetFonts(wxEmptyString, wxEmptyString, f_sizes);
+   #ifdef __WXOSX_COCOA__
+      SetFonts(wxT("Lucida Grande"), wxT("Monaco"), f_sizes);
+   #else
+      SetFonts(wxEmptyString, wxEmptyString, f_sizes);
+   #endif
 }
 
 // -----------------------------------------------------------------------------
@@ -1207,22 +1230,34 @@ void HtmlView::ChangeFontSizes(int size)
 
 void ShowAboutBox()
 {
-   const wxString title = _("About Golly");
-   wxDialog dlg(mainptr, wxID_ANY, title);
+   if (viewptr->waitingforclick) return;
+
+   wxBoxSizer* topsizer = new wxBoxSizer(wxVERTICAL);
+   wxDialog dlg(mainptr, wxID_ANY, wxString(_("About Golly")));
    
    HtmlView* html = new HtmlView(&dlg, wxID_ANY, wxDefaultPosition,
-                                 #ifdef __WXGTK__
+                                 #if wxCHECK_VERSION(2,9,0)
+                                    // work around SetSize bug below!!!
+                                    wxSize(400, 320),
+                                 #elif defined(__WXGTK__)
                                     wxSize(460, 220),
                                  #else
                                     wxSize(386, 220),
                                  #endif
                                  wxHW_SCROLLBAR_NEVER | wxSUNKEN_BORDER);
    html->SetBorders(0);
+   #ifdef __WXOSX_COCOA__
+      html->SetFontSizes(helpfontsize);
+   #endif
    html->CheckAndLoad(_("Help/about.html"));
+   
+   // avoid HtmlView::OnSize calling CheckAndLoad again
+   html->canreload = false;
+   
+   // this call seems to be ignored in __WXOSX_COCOA__!!!
    html->SetSize(html->GetInternalRepresentation()->GetWidth(),
                  html->GetInternalRepresentation()->GetHeight());
    
-   wxBoxSizer* topsizer = new wxBoxSizer(wxVERTICAL);
    topsizer->Add(html, 1, wxALL, 10);
 
    wxButton* okbutt = new wxButton(&dlg, wxID_OK, _("OK"));
@@ -1231,6 +1266,6 @@ void ShowAboutBox()
    
    dlg.SetSizer(topsizer);
    topsizer->Fit(&dlg);
-   dlg.Centre();   
+   dlg.Centre();
    dlg.ShowModal();
 }
