@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -180,16 +180,28 @@ static LayerBar* layerbarptr = NULL;      // global pointer to layer bar
 static wxBitmapButton* bitmapbutt[NUM_BUTTONS] = {NULL};
 static wxToggleButton* togglebutt[MAX_LAYERS] = {NULL};
 
+// width and height of toggle buttons
 const int MAX_TOGGLE_WD = 128;
 const int MIN_TOGGLE_WD = 48;
 #if defined(__WXMSW__)
    const int TOGGLE_HT = 22;
 #elif defined(__WXGTK__)
    const int TOGGLE_HT = 24;
+#elif defined(__WXOSX_COCOA__)
+   const int TOGGLE_HT = 24;
 #else
    const int TOGGLE_HT = 20;
 #endif
 
+// width and height of bitmap buttons
+#if defined(__WXOSX_COCOA__) || defined(__WXGTK__)
+   const int BUTTON_WD = 28;
+   const int BUTTON_HT = 28;
+#else
+   const int BUTTON_WD = 24;
+   const int BUTTON_HT = 24;
+#endif
+
 const wxString SWITCH_LAYER = _("Switch to this layer");
 
 // -----------------------------------------------------------------------------
@@ -238,7 +250,7 @@ LayerBar::LayerBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int ht)
       smallgap = 6;
    #else
       xpos = 4;
-      ypos = 4;
+      ypos = (32 - BUTTON_HT) / 2;
       smallgap = 4;
    #endif
    
@@ -392,12 +404,10 @@ void LayerBar::OnButton(wxCommandEvent& event)
          }
    }
 
-   #ifdef __WXMAC__
-      // avoid weird bug on Mac where viewport can lose keyboard focus after
-      // the user hits DELETE_LAYER button *and* the "All controls" option
-      // is ticked in System Prefs > Keyboard & Mouse > Keyboard Shortcuts
-      viewptr->SetFocus();
-   #endif
+   // avoid weird bug on Mac where viewport can lose keyboard focus after
+   // the user hits DELETE_LAYER button *and* the "All controls" option
+   // is ticked in System Prefs > Keyboard & Mouse > Keyboard Shortcuts
+   viewptr->SetFocus();
 }
 
 // -----------------------------------------------------------------------------
@@ -469,10 +479,10 @@ void LayerBar::OnButtonUp(wxMouseEvent& event)
       wxCommandEvent buttevt(wxEVT_COMMAND_BUTTON_CLICKED, id);
       if (id < MAX_LAYERS) {
          buttevt.SetEventObject(togglebutt[id]);
-         togglebutt[id]->ProcessEvent(buttevt);
+         togglebutt[id]->GetEventHandler()->ProcessEvent(buttevt);
       } else {
          buttevt.SetEventObject(bitmapbutt[id]);
-         bitmapbutt[id]->ProcessEvent(buttevt);
+         bitmapbutt[id]->GetEventHandler()->ProcessEvent(buttevt);
       }
    }
 }
@@ -501,8 +511,8 @@ void LayerBar::AddButton(int id, const wxString& tip)
       if (togglebutt[id] == NULL) {
          Fatal(_("Failed to create layer bar bitmap button!"));
       } else {
-         #ifdef __WXGTK__
-            // nicer to use smaller font on Linux
+         #if defined(__WXGTK__) || defined(__WXOSX_COCOA__)
+            // use smaller font on Linux and OS X Cocoa
             togglebutt[id]->SetWindowVariant(wxWINDOW_VARIANT_SMALL);
          #endif
 
@@ -531,11 +541,11 @@ void LayerBar::AddButton(int id, const wxString& tip)
 
    } else {
       // create bitmap button
-      bitmapbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos));
+      bitmapbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos),
+                                          wxSize(BUTTON_WD, BUTTON_HT));
       if (bitmapbutt[id] == NULL) {
          Fatal(_("Failed to create layer bar bitmap button!"));
       } else {
-         const int BUTTON_WD = 24;        // nominal width of bitmap buttons
          xpos += BUTTON_WD + smallgap;
          
          bitmapbutt[id]->SetToolTip(tip);
@@ -927,25 +937,6 @@ void DestroyTiles()
 
 // -----------------------------------------------------------------------------
 
-void UpdateView()
-{
-   // update main viewport window, including all tile windows if they exist
-   // (tile windows are children of bigview)
-   bigview->Refresh(false);
-   bigview->Update();
-}
-
-// -----------------------------------------------------------------------------
-
-void RefreshView()
-{
-   // refresh main viewport window, including all tile windows if they exist
-   // (tile windows are children of bigview)
-   bigview->Refresh(false);
-}
-
-// -----------------------------------------------------------------------------
-
 void SyncClones()
 {
    if (numclones == 0) return;
@@ -1043,13 +1034,30 @@ void SaveLayerSettings()
 
 // -----------------------------------------------------------------------------
 
+bool RestoreRule(const wxString& rule)
+{
+   const char* err = currlayer->algo->setrule( rule.mb_str(wxConvLocal) );
+   if (err) {
+      // this can happen if the given rule's table/tree file was deleted
+      // or it was edited and some sort of error introduced, so best to
+      // use algo's default rule (which should never fail)
+      currlayer->algo->setrule( currlayer->algo->DefaultRule() );
+      wxString msg = _("The rule \"") + rule;
+      msg += _("\" is no longer valid!\nUsing the default rule instead.");
+      Warning(msg);
+      return false;
+   }
+   return true;
+}
+
+// -----------------------------------------------------------------------------
+
 void CurrentLayerChanged()
 {
    // currlayer has changed since SaveLayerSettings was called;
    // update rule if the new currlayer uses a different algorithm or rule
    if ( currlayer->algtype != oldalgo || !currlayer->rule.IsSameAs(oldrule,false) ) {
-      currlayer->algo->setrule( currlayer->rule.mb_str(wxConvLocal) );
-      // error should never occur here
+      RestoreRule(currlayer->rule);
    }
    
    if (syncviews) currlayer->view->setpositionmag(oldx, oldy, oldmag);
@@ -1988,6 +1996,10 @@ void UpdateCurrentColors()
          // use hexagonal icons
          currlayer->icons15x15 = CopyIcons(hexicons15x15, 15, maxstate);
          currlayer->icons7x7 = CopyIcons(hexicons7x7, 7, maxstate);
+      } else if (currlayer->algo->getgridtype() == lifealgo::VN_GRID) {
+         // use diamond-shaped icons for 4-neighbor von Neumann neighborhood
+         currlayer->icons15x15 = CopyIcons(vnicons15x15, 15, maxstate);
+         currlayer->icons7x7 = CopyIcons(vnicons7x7, 7, maxstate);
       } else {
          // otherwise copy default icons from current algo
          currlayer->icons15x15 = CopyIcons(ad->icons15x15, 15, maxstate);
@@ -2154,7 +2166,7 @@ Layer::Layer()
       // set rule using initrule stored in prefs file
       const char* err = algo->setrule(initrule);
       if (err) {
-         // switch to algo's default rule (user probably edited rule in prefs file)
+         // user might have edited rule in prefs file, or deleted table/tree file
          algo->setrule( algo->DefaultRule() );
       }
    
@@ -2215,8 +2227,11 @@ Layer::Layer()
          algo = CreateNewUniverse(algtype);
          
          // use current rule
-         algo->setrule(currlayer->algo->getrule());
-         // error should never occur here
+         const char* err = algo->setrule(currlayer->algo->getrule());
+         if (err) {
+            // table/tree file might have been deleted
+            algo->setrule( algo->DefaultRule() );
+         }
          
          // initialize undo/redo history
          undoredo = new UndoRedo();
@@ -2591,14 +2606,6 @@ const int HGAP = 12;    // space left and right of vertically stacked boxes
    const int STDHGAP = 10;
 #endif
 
-#if defined(__WXMAC__) && wxCHECK_VERSION(2,8,0)
-   // fix wxALIGN_CENTER_VERTICAL bug in wxMac 2.8.0+;
-   // only happens when a wxStaticText/wxButton box is next to a wxChoice box
-   #define FIX_ALIGN_BUG wxBOTTOM,4
-#else
-   #define FIX_ALIGN_BUG wxALL,0
-#endif
-
 const int BITMAP_WD = 60;     // width of bitmap in color buttons
 const int BITMAP_HT = 20;     // height of bitmap in color buttons
 
@@ -2718,7 +2725,12 @@ void ColorDialog::AddColorButton(wxWindow* parent, wxBoxSizer* hbox, int id, wxC
    FillRect(dc, rect, brush);
    dc.SelectObject(wxNullBitmap);
    
-   wxBitmapButton* bb = new wxBitmapButton(parent, id, bitmap, wxPoint(0,0));
+   wxBitmapButton* bb = new wxBitmapButton(parent, id, bitmap, wxPoint(0,0),
+                                           #if defined(__WXOSX_COCOA__)
+                                              wxSize(BITMAP_WD + 12, BITMAP_HT + 12));
+                                           #else
+                                              wxDefaultSize);
+                                           #endif
    if (bb) hbox->Add(bb, 0, wxALIGN_CENTER_VERTICAL, 0);
 }
 
