@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -72,14 +72,29 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // avoid warning about _ being redefined
 #undef _
 
-// on Windows, wxWidgets redefines uid_t/gid_t which breaks Perl's typedefs:
-#undef uid_t
-#undef gid_t
+#ifdef __WXMSW__
+   // on Windows, wxWidgets defines uid_t/gid_t which breaks Perl's typedefs:
+   #undef uid_t
+   #undef gid_t
+   // can't do "#undef mode_t" for a typedef so use this hack:
+   typedef unsigned short MODE1;  // from C:\Perl\lib\CORE\win32.h
+   typedef int MODE2;             // from C:\wxWidgets\include\wx\filefn.h
+   #define mode_t MODE1
+#endif
 
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
 
+#ifdef __WXMSW__
+   #undef mode_t
+   #define mode_t MODE2
+#endif
+
+// restore wxWidgets definition for _ (from include/wx/intl.h)
+#undef _
+#define _(s) wxGetTranslation(_T(s))
+
 /*
  * Quoting Jan Dubois of Active State:
  *    ActivePerl build 822 still identifies itself as 5.8.8 but already
@@ -116,9 +131,16 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
    #define PERL5101_OR_LATER
 #endif
 
-// restore wxWidgets definition for _ (from include/wx/intl.h)
-#undef _
-#define _(s) wxGetTranslation(_T(s))
+// check if we're building with Perl 5.14 or later
+#if (PERL_REVISION == 5) && (PERL_VERSION >= 14)
+   #define PERL514_OR_LATER
+#endif
+
+// Check if PL_thr_key is a real variable or instead a macro which calls
+// Perl_Gthr_key_ptr(NULL), which was the default before Perl 5.14:
+#ifdef PL_thr_key
+   #define PERL_THR_KEY_FUNC 1
+#endif
 
 static PerlInterpreter* my_perl = NULL;
 
@@ -144,7 +166,11 @@ EXTERN_C void boot_DynaLoader(pTHX_ CV* cv);
 extern "C"
 {
 #ifdef USE_ITHREADS
-   perl_key*(*G_Perl_Gthr_key_ptr)(register PerlInterpreter*);
+   #ifdef PERL_THR_KEY_FUNC
+      perl_key*(*G_Perl_Gthr_key_ptr)(register PerlInterpreter*);
+   #else
+      perl_key *G_PL_thr_key;
+   #endif
 #endif
    SV**(*G_Perl_av_fetch)(pTHX_ AV*, I32, I32);
    I32(*G_Perl_av_len)(pTHX_ AV*);
@@ -204,7 +230,13 @@ extern "C"
 }
 
 // redefine Perl functions to their equivalent G_* wrappers
-#define Perl_Gthr_key_ptr        G_Perl_Gthr_key_ptr
+#ifdef USE_ITHREADS
+   #ifdef PERL_THR_KEY_FUNC
+      #define Perl_Gthr_key_ptr        G_Perl_Gthr_key_ptr
+   #else
+      #define PL_thr_key               (*G_PL_thr_key)
+   #endif
+#endif
 #define Perl_av_fetch            G_Perl_av_fetch
 #define Perl_av_len              G_Perl_av_len
 #define Perl_av_push             G_Perl_av_push
@@ -276,7 +308,11 @@ static struct PerlFunc
 } perlFuncs[] =
 {
 #ifdef USE_ITHREADS
-   PERL_FUNC(Perl_Gthr_key_ptr)
+   #ifdef PERL_THR_KEY_FUNC
+      PERL_FUNC(Perl_Gthr_key_ptr)
+   #else
+      PERL_FUNC(PL_thr_key)
+   #endif
 #endif
    PERL_FUNC(Perl_av_fetch)
    PERL_FUNC(Perl_av_len)
@@ -307,19 +343,24 @@ static struct PerlFunc
    PERL_FUNC(Perl_sys_term)
 #endif
 #ifdef MULTIPLICITY
-   #ifdef PERL510_OR_LATER
-      PERL_FUNC(Perl_Imarkstack_ptr_ptr)
-      PERL_FUNC(Perl_Istack_base_ptr)
-      PERL_FUNC(Perl_Istack_max_ptr)
-      PERL_FUNC(Perl_Istack_sp_ptr)
-   #else
-      PERL_FUNC(Perl_Tmarkstack_ptr_ptr)
-      PERL_FUNC(Perl_Tstack_base_ptr)
-      PERL_FUNC(Perl_Tstack_max_ptr)
-      PERL_FUNC(Perl_Tstack_sp_ptr)
+   #ifndef PERL514_OR_LATER
+      // before Perl 5.14:
+      PERL_FUNC(Perl_Iexit_flags_ptr)
+      PERL_FUNC(Perl_Iperl_destruct_level_ptr)
+      #ifdef PERL510_OR_LATER
+         // Perl 5.10/5.12 only:
+         PERL_FUNC(Perl_Imarkstack_ptr_ptr)
+         PERL_FUNC(Perl_Istack_base_ptr)
+         PERL_FUNC(Perl_Istack_max_ptr)
+         PERL_FUNC(Perl_Istack_sp_ptr)
+      #else
+         // before Perl 5.10:
+         PERL_FUNC(Perl_Tmarkstack_ptr_ptr)
+         PERL_FUNC(Perl_Tstack_base_ptr)
+         PERL_FUNC(Perl_Tstack_max_ptr)
+         PERL_FUNC(Perl_Tstack_sp_ptr)
+      #endif
    #endif
-   PERL_FUNC(Perl_Iexit_flags_ptr)
-   PERL_FUNC(Perl_Iperl_destruct_level_ptr)
 #else  /* no MULTIPLICITY */
    /* N.B. these are actually variables, not functions, but the distinction does
       not matter for symbol resolution: */
@@ -671,10 +712,6 @@ XS(pl_load)
    // create temporary universe of same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
    // readpattern will call setrule
-   // tempalgo->setrule(currlayer->algo->getrule());
-
-   // readpattern might change rule
-   wxString oldrule = wxString(currlayer->algo->getrule(), wxConvLocal);
 
    // read pattern into temporary universe
    const char* err = readpattern(FILENAME, *tempalgo);
@@ -690,9 +727,6 @@ XS(pl_load)
       }
    }
 
-   // restore rule
-   currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
-
    if (err) {
       delete tempalgo;
       PERL_ERROR(err);
@@ -731,7 +765,8 @@ XS(pl_store)
 
    // create temporary universe of same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
-   tempalgo->setrule(currlayer->algo->getrule());
+   const char* err = tempalgo->setrule(currlayer->algo->getrule());
+   if (err) tempalgo->setrule(tempalgo->DefaultRule());
 
    // copy cell array into temporary universe
    bool multistate = ((av_len(inarray) + 1) & 1) == 1;
@@ -768,7 +803,7 @@ XS(pl_store)
    pattern_format format = savexrle ? XRLE_format : RLE_format;
    // if grid is bounded then force XRLE_format so that position info is recorded
    if (tempalgo->gridwd > 0 || tempalgo->gridht > 0) format = XRLE_format;
-   const char* err = writepattern(FILENAME, *tempalgo, format,
+   err = writepattern(FILENAME, *tempalgo, format, no_compression,
                         top.toint(), left.toint(), bottom.toint(), right.toint());
    delete tempalgo;
    if (err) PERL_ERROR(err);
@@ -1217,7 +1252,8 @@ XS(pl_evolve)
 
    // create a temporary universe of same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
-   tempalgo->setrule(currlayer->algo->getrule());
+   const char* err = tempalgo->setrule(currlayer->algo->getrule());
+   if (err) tempalgo->setrule(tempalgo->DefaultRule());
 
    // copy cell array into temporary universe
    bool multistate = ((av_len(inarray) + 1) & 1) == 1;
@@ -1273,7 +1309,7 @@ XS(pl_evolve)
 
    // convert new pattern into a new cell array
    AV* outarray = (AV*)sv_2mortal( (SV*)newAV() );
-   const char* err = ExtractCellArray(outarray, tempalgo);
+   err = ExtractCellArray(outarray, tempalgo);
    delete tempalgo;
    if (err) PERL_ERROR(err);
 
@@ -1640,13 +1676,15 @@ XS(pl_getclip)
    // create a temporary universe for storing clipboard pattern;
    // GetClipboardPattern assumes it is same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
-   tempalgo->setrule(currlayer->algo->getrule());
+   const char* err = tempalgo->setrule(currlayer->algo->getrule());
+   if (err) tempalgo->setrule(tempalgo->DefaultRule());
 
    // read clipboard pattern into temporary universe and set edges
    // (not a minimal bounding box if pattern is empty or has empty borders)
    bigint top, left, bottom, right;
    if ( viewptr->GetClipboardPattern(&tempalgo, &top, &left, &bottom, &right) ) {
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
+         delete tempalgo;
          PERL_ERROR("g_getclip error: pattern is too big.");
       }
       int itop = top.toint();
@@ -2842,6 +2880,59 @@ XS(pl_getstring)
 
 // -----------------------------------------------------------------------------
 
+XS(pl_getxy)
+{
+   IGNORE_UNUSED_PARAMS;
+   RETURN_IF_ABORTED;
+   dXSARGS;
+   if (items != 0) PERL_ERROR("Usage: $string = g_getxy().");
+
+   statusptr->CheckMouseLocation(mainptr->IsActive());   // sets mousepos
+   if (viewptr->showcontrols) mousepos = wxEmptyString;
+
+   XSRETURN_PV((const char*)mousepos.mb_str(wxConvLocal));
+}
+
+// -----------------------------------------------------------------------------
+
+XS(pl_getevent)
+{
+   IGNORE_UNUSED_PARAMS;
+   RETURN_IF_ABORTED;
+   dXSARGS;
+   if (items > 1) PERL_ERROR("Usage: $string = g_getevent($get=1).");
+
+   int get = 1;
+   if (items > 0) get = SvIV(ST(0));
+
+   wxString event;
+   GSF_getevent(event, get);
+
+   XSRETURN_PV((const char*)event.mb_str(wxConvLocal));
+}
+
+// -----------------------------------------------------------------------------
+
+XS(pl_doevent)
+{
+   IGNORE_UNUSED_PARAMS;
+   RETURN_IF_ABORTED;
+   dXSARGS;
+   if (items != 1) PERL_ERROR("Usage: g_doevent($string).");
+
+   STRLEN n_a;
+   char* event = SvPV(ST(0), n_a);
+
+   if (event[0]) {
+      const char* err = GSF_doevent(wxString(event,wxConvLocal));
+      if (err) PERL_ERROR(err);
+   }
+
+   XSRETURN(0);
+}
+
+// -----------------------------------------------------------------------------
+
 XS(pl_getkey)
 {
    IGNORE_UNUSED_PARAMS;
@@ -3015,9 +3106,14 @@ XS(pl_fatal)
    STRLEN n_a;
    char* err = SvPV(ST(0),n_a);
 
-   // store message in global string (shown after script finishes)
-   scripterr = wxString(err, wxConvLocal);
-
+   if (scripterr == wxString(abortmsg,wxConvLocal)) {
+      // this can happen in Perl 5.14 so don't change scripterr
+      // otherwise a message box will appear
+   } else {
+      // store message in global string (shown after script finishes)
+      scripterr = wxString(err, wxConvLocal);
+   }
+   
    XSRETURN(0);
 }
 
@@ -3152,6 +3248,10 @@ EXTERN_C void xs_init(pTHX)
    newXS((char*)"g_setclipstr",   pl_setclipstr,   (char*)file);
    newXS((char*)"g_getclipstr",   pl_getclipstr,   (char*)file);
    newXS((char*)"g_getstring",    pl_getstring,    (char*)file);
+   newXS((char*)"g_getxy",        pl_getxy,        (char*)file);
+   newXS((char*)"g_getevent",     pl_getevent,     (char*)file);
+   newXS((char*)"g_doevent",      pl_doevent,      (char*)file);
+   // next two are deprecated (use g_getevent and g_doevent)
    newXS((char*)"g_getkey",       pl_getkey,       (char*)file);
    newXS((char*)"g_dokey",        pl_dokey,        (char*)file);
    newXS((char*)"g_show",         pl_show,         (char*)file);
