@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -74,10 +74,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // so Golly will start up even if Python isn't installed.
 // Based on code from Mahogany (mahogany.sourceforge.net) and Vim (www.vim.org).
 
-// wxMac bug??? why does dynlib.Load fail if given
-// "/System/Library/Frameworks/Python.framework/Versions/2.3/Python"???
-// #if 1
-#ifndef __WXMAC__
+#ifdef __WXMAC__
+   // avoid warnings (only if using CMake, for some unknown reason)
+   #undef SIZEOF_LONG
+   #undef SIZEOF_SIZE_T
+   #undef SIZEOF_VOID_P
+#else
    // load Python lib at runtime
    #define USE_PYTHON_DYNAMIC
 
@@ -86,11 +88,23 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
       #undef _POSIX_C_SOURCE
    #endif
 
+   #ifdef __WXMSW__
+      // avoid warning on Windows
+      #undef HAVE_SSIZE_T
+   #endif
+
    // prevent Python.h from adding Python library to link settings
    #define USE_DL_EXPORT
 #endif
 
+// workaround for hard-linking to missing python<ver>_d.lib
+#ifdef _DEBUG
+#undef _DEBUG
 #include <Python.h>
+#define _DEBUG
+#else
+#include <Python.h>
+#endif
 
 #ifdef USE_PYTHON_DYNAMIC
 
@@ -594,10 +608,6 @@ static PyObject* py_load(PyObject* self, PyObject* args)
    // create temporary universe of same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
    // readpattern will call setrule
-   // tempalgo->setrule(currlayer->algo->getrule());
-
-   // readpattern might change rule
-   wxString oldrule = wxString(currlayer->algo->getrule(), wxConvLocal);
 
    // read pattern into temporary universe
    const char* err = readpattern(FILENAME, *tempalgo);
@@ -613,9 +623,6 @@ static PyObject* py_load(PyObject* self, PyObject* args)
       }
    }
 
-   // restore rule
-   currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
-
    if (err) {
       delete tempalgo;
       PYTHON_ERROR(err);
@@ -648,7 +655,8 @@ static PyObject* py_store(PyObject* self, PyObject* args)
 
    // create temporary universe of same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
-   tempalgo->setrule(currlayer->algo->getrule());
+   const char* err = tempalgo->setrule(currlayer->algo->getrule());
+   if (err) tempalgo->setrule(tempalgo->DefaultRule());
 
    // copy cell list into temporary universe
    bool multistate = (PyList_Size(inlist) & 1) == 1;
@@ -685,7 +693,7 @@ static PyObject* py_store(PyObject* self, PyObject* args)
    pattern_format format = savexrle ? XRLE_format : RLE_format;
    // if grid is bounded then force XRLE_format so that position info is recorded
    if (tempalgo->gridwd > 0 || tempalgo->gridht > 0) format = XRLE_format;
-   const char* err = writepattern(FILENAME, *tempalgo, format,
+   err = writepattern(FILENAME, *tempalgo, format, no_compression,
                         top.toint(), left.toint(), bottom.toint(), right.toint());
    delete tempalgo;
    if (err) PYTHON_ERROR(err);
@@ -1091,7 +1099,8 @@ static PyObject* py_evolve(PyObject* self, PyObject* args)
 
    // create a temporary universe of same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
-   tempalgo->setrule(currlayer->algo->getrule());
+   const char* err = tempalgo->setrule(currlayer->algo->getrule());
+   if (err) tempalgo->setrule(tempalgo->DefaultRule());
 
    // copy cell list into temporary universe
    bool multistate = (PyList_Size(inlist) & 1) == 1;
@@ -1513,13 +1522,15 @@ static PyObject* py_getclip(PyObject* self, PyObject* args)
    // create a temporary universe for storing clipboard pattern;
    // GetClipboardPattern assumes it is same type as current universe
    lifealgo* tempalgo = CreateNewUniverse(currlayer->algtype, allowcheck);
-   tempalgo->setrule(currlayer->algo->getrule());
+   const char* err = tempalgo->setrule(currlayer->algo->getrule());
+   if (err) tempalgo->setrule(tempalgo->DefaultRule());
 
    // read clipboard pattern into temporary universe and set edges
    // (not a minimal bounding box if pattern is empty or has empty borders)
    bigint top, left, bottom, right;
    if ( viewptr->GetClipboardPattern(&tempalgo, &top, &left, &bottom, &right) ) {
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
+         delete tempalgo;
          Py_DECREF(outlist);
          PYTHON_ERROR("getclip error: pattern is too big.");
       }
@@ -2647,6 +2658,55 @@ static PyObject* py_getstring(PyObject* self, PyObject* args)
 
 // -----------------------------------------------------------------------------
 
+static PyObject* py_getxy(PyObject* self, PyObject* args)
+{
+   if (PythonScriptAborted()) return NULL;
+   wxUnusedVar(self);
+
+   if (!PyArg_ParseTuple(args, (char*)"")) return NULL;
+
+   statusptr->CheckMouseLocation(mainptr->IsActive());   // sets mousepos
+   if (viewptr->showcontrols) mousepos = wxEmptyString;
+   
+   return Py_BuildValue((char*)"s", (const char*)mousepos.mb_str(wxConvLocal));
+}
+
+// -----------------------------------------------------------------------------
+
+static PyObject* py_getevent(PyObject* self, PyObject* args)
+{
+   if (PythonScriptAborted()) return NULL;
+   wxUnusedVar(self);
+   int get = 1;
+
+   if (!PyArg_ParseTuple(args, (char*)"|i", &get)) return NULL;
+
+   wxString event;
+   GSF_getevent(event, get);
+
+   return Py_BuildValue((char*)"s", (const char*)event.mb_str(wxConvLocal));
+}
+
+// -----------------------------------------------------------------------------
+
+static PyObject* py_doevent(PyObject* self, PyObject* args)
+{
+   if (PythonScriptAborted()) return NULL;
+   wxUnusedVar(self);
+   char* event;
+
+   if (!PyArg_ParseTuple(args, (char*)"s", &event)) return NULL;
+
+   if (event[0]) {
+      const char* err = GSF_doevent(wxString(event,wxConvLocal));
+      if (err) PYTHON_ERROR(err);
+   }
+
+   RETURN_NONE;
+}
+
+// -----------------------------------------------------------------------------
+
 static PyObject* py_getkey(PyObject* self, PyObject* args)
 {
    if (PythonScriptAborted()) return NULL;
@@ -2666,7 +2726,7 @@ static PyObject* py_dokey(PyObject* self, PyObject* args)
 {
    if (PythonScriptAborted()) return NULL;
    wxUnusedVar(self);
-   char* ascii = 0;
+   char* ascii;
 
    if (!PyArg_ParseTuple(args, (char*)"s", &ascii)) return NULL;
 
@@ -2905,6 +2965,10 @@ static PyMethodDef py_methods[] = {
    { "setclipstr",   py_setclipstr, METH_VARARGS, "set the clipboard contents to a given string value" },
    { "getclipstr",   py_getclipstr, METH_VARARGS, "retrieve the contents of the clipboard as a string" },
    { "getstring",    py_getstring,  METH_VARARGS, "display dialog box to get string from user" },
+   { "getxy",        py_getxy,      METH_VARARGS, "return current grid location of mouse" },
+   { "getevent",     py_getevent,   METH_VARARGS, "return keyboard/mouse event or empty string if none" },
+   { "doevent",      py_doevent,    METH_VARARGS, "pass given keyboard/mouse event to Golly to handle" },
+   // next two are deprecated (use getevent and doevent)
    { "getkey",       py_getkey,     METH_VARARGS, "return key hit by user or empty string if none" },
    { "dokey",        py_dokey,      METH_VARARGS, "pass given key to Golly's standard key handler" },
    { "show",         py_show,       METH_VARARGS, "show given string in status bar" },
@@ -2972,55 +3036,12 @@ bool InitPython()
       if ( PyRun_SimpleString(command.mb_str(wxConvLocal)) < 0 )
          Warning(_("Failed to append Scripts path!"));
 
-      // nicer to reload all modules in case changes were made by user;
-      // code comes from http://pyunit.sourceforge.net/notes/reloading.html
-      /* unfortunately it causes an AttributeError
-      if ( PyRun_SimpleString(
-            "import __builtin__\n"
-            "class RollbackImporter:\n"
-            "   def __init__(self):\n"
-            "      self.previousModules = sys.modules.copy()\n"
-            "      self.realImport = __builtin__.__import__\n"
-            "      __builtin__.__import__ = self._import\n"
-            "      self.newModules = {}\n"
-            "   def _import(self, name, globals=None, locals=None, fromlist=[]):\n"
-            "      result = apply(self.realImport, (name, globals, locals, fromlist))\n"
-            "      self.newModules[name] = 1\n"
-            "      return result\n"
-            "   def uninstall(self):\n"
-            "      for modname in self.newModules.keys():\n"
-            "         if not self.previousModules.has_key(modname):\n"
-            "            del(sys.modules[modname])\n"
-            "      __builtin__.__import__ = self.realImport\n"
-            "rollbackImporter = RollbackImporter()\n"
-            ) < 0
-         ) Warning(_("RollbackImporter code failed!"));
-      */
-
       pyinited = true;
    } else {
-      // Py_Initialize has already been successfully called
-      if ( PyRun_SimpleString(
-            // Py_Finalize is not used to close stderr so reset it here
-            "sys.stderr.data = ''\n"
-
-            // reload all modules in case changes were made by user
-            /* this almost works except for strange error the 2nd time we run gun-demo.py
-            "import sys\n"
-            "for m in sys.modules.keys():\n"
-            "   t = str(type(sys.modules[m]))\n"
-            "   if t.find('module') < 0 or m == 'golly' or m == 'sys' or m[0] == '_':\n"
-            "      pass\n"
-            "   else:\n"
-            "      reload(sys.modules[m])\n"
-            */
-
-            /* RollbackImporter code causes an error
-            "if rollbackImporter: rollbackImporter.uninstall()\n"
-            "rollbackImporter = RollbackImporter()\n"
-            */
-            ) < 0
-         ) Warning(_("PyRun_SimpleString failed!"));
+      // Py_Initialize has already been successfully called;
+      // Py_Finalize is not used to close stderr so reset it here
+      if ( PyRun_SimpleString("sys.stderr.data = ''\n") < 0 )
+         Warning(_("PyRun_SimpleString failed!"));
    }
 
    return true;
