@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -202,7 +202,9 @@ void CreateTranslucentControls()
       wxAlphaPixelData data(*ctrlsbitmap, wxPoint(0,0), wxSize(controlswd,controlsht));
       if (data) {
          int alpha = 192;     // 75% opaque
-         data.UseAlpha();
+         #if !wxCHECK_VERSION(2,9,0)
+            data.UseAlpha();
+         #endif
          wxAlphaPixelData::Iterator p(data);
          for ( int y = 0; y < controlsht; y++ ) {
             wxAlphaPixelData::Iterator rowstart = p;
@@ -218,8 +220,8 @@ void CreateTranslucentControls()
                   p.Alpha() = 0;
                } else {
                   // make all non-black pixels translucent
-                  #ifdef __WXMSW__
-                     // premultiply the RGB values on Windows
+                  #if defined(__WXMSW__) || (defined(__WXMAC__) && wxCHECK_VERSION(2,8,8))
+                     // premultiply the RGB values
                      p.Red()   = r * alpha / 255;
                      p.Green() = g * alpha / 255;
                      p.Blue()  = b * alpha / 255;
@@ -248,7 +250,9 @@ void CreateTranslucentControls()
       if (data) {
          int alpha = 128;     // 50% opaque
          int gray = 20;       // very dark gray
-         data.UseAlpha();
+         #if !wxCHECK_VERSION(2,9,0)
+            data.UseAlpha();
+         #endif
          wxAlphaPixelData::Iterator p(data);
          for ( int y = 0; y < controlsht; y++ ) {
             wxAlphaPixelData::Iterator rowstart = p;
@@ -264,8 +268,8 @@ void CreateTranslucentControls()
                   p.Alpha() = 0;
                } else {
                   // make all non-black pixels translucent gray
-                  #ifdef __WXMSW__
-                     // premultiply the RGB values on Windows
+                  #if defined(__WXMSW__) || (defined(__WXMAC__) && wxCHECK_VERSION(2,8,8))
+                     // premultiply the RGB values
                      p.Red()   = gray * alpha / 255;
                      p.Green() = gray * alpha / 255;
                      p.Blue()  = gray * alpha / 255;
@@ -328,6 +332,7 @@ void DrawControls(wxDC& dc, wxRect& rect)
          wxMemoryDC memdc;
          memdc.SelectObject(*ctrlsbitmap);
          dc.Blit(rect.x, rect.y, rect.width, rect.height, &memdc, 0, 0, wxCOPY, true);
+         memdc.SelectObject(wxNullBitmap);
       #endif
       
       if (currcontrol > NO_CONTROL && darkctrls) {
@@ -353,6 +358,7 @@ void DrawControls(wxDC& dc, wxRect& rect)
             wxMemoryDC memdc;
             memdc.SelectObject(*darkctrls);
             dc.Blit(rect.x + x, rect.y + y, buttsize, buttsize, &memdc, x, y, wxCOPY, true);
+            memdc.SelectObject(wxNullBitmap);
          #endif
       }
    }
@@ -367,8 +373,8 @@ void SetSelectionPixels(wxBitmap* bitmap, const wxColor* color)
    if (data) {
       int alpha = 128;     // 50% opaque
       
-      #ifdef __WXMSW__
-         // premultiply the RGB values on Windows
+      #if defined(__WXMSW__) || (defined(__WXMAC__) && wxCHECK_VERSION(2,8,8))
+         // premultiply the RGB values
          int r = color->Red() * alpha / 255;
          int g = color->Green() * alpha / 255;
          int b = color->Blue() * alpha / 255;
@@ -378,7 +384,9 @@ void SetSelectionPixels(wxBitmap* bitmap, const wxColor* color)
          int b = color->Blue();
       #endif
       
-      data.UseAlpha();
+      #if !wxCHECK_VERSION(2,9,0)
+         data.UseAlpha();
+      #endif
       wxAlphaPixelData::Iterator p(data);
       for ( int y = 0; y < selht; y++ ) {
          wxAlphaPixelData::Iterator rowstart = p;
@@ -435,8 +443,37 @@ void DestroyDrawingData()
 
 // -----------------------------------------------------------------------------
 
+// called from wx_render::pixblit to draw a bitmap at 1:1 scale
+void DrawPixmap(unsigned char* byteptr, int x, int y, int w, int h, int stride)
+{
+   wxAlphaPixelData pxldata(*pixmap);
+   if (pxldata) {
+      #if defined(__WXGTK__) && !wxCHECK_VERSION(2,9,0)
+         pxldata.UseAlpha();
+      #endif
+      wxAlphaPixelData::Iterator p(pxldata);
+      for ( int row = 0; row < h; row++ ) {
+         wxAlphaPixelData::Iterator rowstart = p;
+         for ( int col = 0; col < w; col++ ) {
+            p.Red()   = byteptr[(row*stride + col)*3 + 0];
+            p.Green() = byteptr[(row*stride + col)*3 + 1];
+            p.Blue()  = byteptr[(row*stride + col)*3 + 2];
+            #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
+               p.Alpha() = 255;
+            #endif
+            p++;
+         }
+         p = rowstart;
+         p.OffsetY(pxldata, 1);
+      }
+   }
+   currdc->DrawBitmap(*pixmap, x, y);
+}
+
+// -----------------------------------------------------------------------------
+
 // called from wx_render::pixblit to magnify given pixmap by pmscale (2, 4, ... 2^MAX_MAG)
-void DrawStretchedPixmap(unsigned char* byteptr, int x, int y, int w, int h, int pmscale)
+void DrawStretchedPixmap(unsigned char* byteptr, int x, int y, int w, int h, int pmscale, int stride)
 {
    int cellsize = pmscale > 2 ? pmscale - 1 : pmscale;
    bool drawgap = (pmscale > 2 && pmscale < (1 << mingridmag)) ||
@@ -455,65 +492,57 @@ void DrawStretchedPixmap(unsigned char* byteptr, int x, int y, int w, int h, int
    // might be faster to draw rectangles above certain scales???
    wxAlphaPixelData pxldata(*pixmap);
    if (pxldata) {
-      #ifdef __WXGTK__
+      #if defined(__WXGTK__) && !wxCHECK_VERSION(2,9,0)
          pxldata.UseAlpha();
       #endif
       wxAlphaPixelData::Iterator p(pxldata);
       for ( int row = 0; row < h; row++ ) {
          wxAlphaPixelData::Iterator rowstart = p;
          for ( int col = 0; col < w; col++ ) {
-            int newx = x + col * pmscale;
-            int newy = y + row * pmscale;
-            if (newx < 0 || newy < 0 || newx >= currwd || newy >= currht) {
-               // clip cell outside viewport
-            } else {
-               unsigned char state = *byteptr;
-               unsigned char r = currlayer->cellr[state];
-               unsigned char g = currlayer->cellg[state];
-               unsigned char b = currlayer->cellb[state];
-               
-               // expand byte into cellsize*cellsize pixels
-               wxAlphaPixelData::Iterator topleft = p;
-               for (int i = 0; i < cellsize; i++) {
-                  wxAlphaPixelData::Iterator colstart = p;
-                  for (int j = 0; j < cellsize; j++) {
-                     p.Red()   = r;
-                     p.Green() = g;
-                     p.Blue()  = b;
-                     #ifdef __WXGTK__
-                        p.Alpha() = 255;
-                     #endif
-                     p++;
-                  }
-                  if (drawgap) {
-                     // draw dead pixels at right edge of cell
-                     p.Red()   = deadred;
-                     p.Green() = deadgreen;
-                     p.Blue()  = deadblue;
-                     #ifdef __WXGTK__
-                        p.Alpha() = 255;
-                     #endif
-                  }
-                  p = colstart;
-                  p.OffsetY(pxldata, 1);
+            unsigned char state = byteptr[row*stride + col];
+            unsigned char r = currlayer->cellr[state];
+            unsigned char g = currlayer->cellg[state];
+            unsigned char b = currlayer->cellb[state];
+
+            // expand byte into cellsize*cellsize pixels
+            wxAlphaPixelData::Iterator topleft = p;
+            for (int i = 0; i < cellsize; i++) {
+               wxAlphaPixelData::Iterator colstart = p;
+               for (int j = 0; j < cellsize; j++) {
+                  p.Red()   = r;
+                  p.Green() = g;
+                  p.Blue()  = b;
+                  #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
+                     p.Alpha() = 255;
+                  #endif
+                  p++;
                }
                if (drawgap) {
-                  // draw dead pixels at bottom edge of cell
-                  for (int j = 0; j <= cellsize; j++) {
-                     p.Red()   = deadred;
-                     p.Green() = deadgreen;
-                     p.Blue()  = deadblue;
-                     #ifdef __WXGTK__
-                        p.Alpha() = 255;
-                     #endif
-                     p++;
-                  }
+                  // draw dead pixels at right edge of cell
+                  p.Red()   = deadred;
+                  p.Green() = deadgreen;
+                  p.Blue()  = deadblue;
+                  #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
+                     p.Alpha() = 255;
+                  #endif
                }
-               p = topleft;
+               p = colstart;
+               p.OffsetY(pxldata, 1);
             }
+            if (drawgap) {
+               // draw dead pixels at bottom edge of cell
+               for (int j = 0; j <= cellsize; j++) {
+                  p.Red()   = deadred;
+                  p.Green() = deadgreen;
+                  p.Blue()  = deadblue;
+                  #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
+                     p.Alpha() = 255;
+                  #endif
+                  p++;
+               }
+            }
+            p = topleft;
             p.OffsetX(pxldata, pmscale);
-            
-            byteptr++;        // move to next byte in pmdata
          }
          p = rowstart;
          p.OffsetY(pxldata, pmscale);
@@ -524,7 +553,7 @@ void DrawStretchedPixmap(unsigned char* byteptr, int x, int y, int w, int h, int
 
 // -----------------------------------------------------------------------------
 
-void DrawIcons(unsigned char* byteptr, int x, int y, int w, int h, int pmscale)
+void DrawIcons(unsigned char* byteptr, int x, int y, int w, int h, int pmscale, int stride)
 {
    // called from wx_render::pixblit to draw icons for each live cell;
    // assume pmscale > 2 (should be 8 or 16)
@@ -544,127 +573,84 @@ void DrawIcons(unsigned char* byteptr, int x, int y, int w, int h, int pmscale)
 
    wxAlphaPixelData pxldata(*pixmap);
    if (pxldata) {
-      #ifdef __WXGTK__
+      #if defined(__WXGTK__) && !wxCHECK_VERSION(2,9,0)
          pxldata.UseAlpha();
       #endif
       wxAlphaPixelData::Iterator p(pxldata);
       for ( int row = 0; row < h; row++ ) {
          wxAlphaPixelData::Iterator rowstart = p;
          for ( int col = 0; col < w; col++ ) {
-            int newx = x + col * pmscale;
-            int newy = y + row * pmscale;
-            if (newx < 0 || newy < 0 || newx >= currwd || newy >= currht) {
-               // clip cell outside viewport
-            } else {
-               unsigned char state = *byteptr;
-               
-               wxAlphaPixelData::Iterator topleft = p;
-               if (state && iconmaps[state]) {
-                  // copy cellsize*cellsize pixels from iconmaps[state]
-                  // but convert black pixels to dead cell color
-                  bool multicolor = iconmaps[1]->GetDepth() > 1;
+            unsigned char state = byteptr[row*stride + col];
+
+            wxAlphaPixelData::Iterator topleft = p;
+            if (state && iconmaps[state]) {
+               // copy cellsize*cellsize pixels from iconmaps[state]
+               // but convert black pixels to dead cell color
+               bool multicolor = iconmaps[1]->GetDepth() > 1;
+               #ifdef __WXMSW__
+                  // must use wxNativePixelData for bitmaps with no alpha channel
+                  wxNativePixelData icondata(*iconmaps[state]);
+               #else
+                  wxAlphaPixelData icondata(*iconmaps[state]);
+               #endif
+               if (icondata) {
                   #ifdef __WXMSW__
-                     // must use wxNativePixelData for bitmaps with no alpha channel
-                     wxNativePixelData icondata(*iconmaps[state]);
+                     wxNativePixelData::Iterator iconpxl(icondata);
                   #else
-                     wxAlphaPixelData icondata(*iconmaps[state]);
+                     wxAlphaPixelData::Iterator iconpxl(icondata);
                   #endif
-                  if (icondata) {
-                     #ifdef __WXMSW__
-                        wxNativePixelData::Iterator iconpxl(icondata);
-                     #else
-                        wxAlphaPixelData::Iterator iconpxl(icondata);
-                     #endif
-                     for (int i = 0; i < cellsize; i++) {
-                        wxAlphaPixelData::Iterator colstart = p;
-                        #ifdef __WXMSW__
-                           wxNativePixelData::Iterator iconrow = iconpxl;
-                        #else
-                           wxAlphaPixelData::Iterator iconrow = iconpxl;
-                        #endif
-                        for (int j = 0; j < cellsize; j++) {
-                           if (iconpxl.Red() || iconpxl.Green() || iconpxl.Blue()) {
-                              if (multicolor) {
-                                 // use non-black pixel in multi-colored icon
-                                 if (swapcolors) {
-                                    p.Red()   = 255 - iconpxl.Red();
-                                    p.Green() = 255 - iconpxl.Green();
-                                    p.Blue()  = 255 - iconpxl.Blue();
-                                 } else {
-                                    p.Red()   = iconpxl.Red();
-                                    p.Green() = iconpxl.Green();
-                                    p.Blue()  = iconpxl.Blue();
-                                 }
-                              } else {
-                                 // replace non-black pixel with current cell color
-                                 p.Red()   = currlayer->cellr[state];
-                                 p.Green() = currlayer->cellg[state];
-                                 p.Blue()  = currlayer->cellb[state];
-                              }
-                           } else {
-                              // replace black pixel with dead cell color
-                              p.Red()   = deadred;
-                              p.Green() = deadgreen;
-                              p.Blue()  = deadblue;
-                           }
-                           #ifdef __WXGTK__
-                              p.Alpha() = 255;
-                           #endif
-                           p++;
-                           iconpxl++;
-                        }
-                        if (drawgap) {
-                           // draw dead pixels at right edge of cell
-                           p.Red()   = deadred;
-                           p.Green() = deadgreen;
-                           p.Blue()  = deadblue;
-                           #ifdef __WXGTK__
-                              p.Alpha() = 255;
-                           #endif
-                        }
-                        p = colstart;
-                        p.OffsetY(pxldata, 1);
-                        // move to next row of icon bitmap
-                        iconpxl = iconrow;
-                        iconpxl.OffsetY(icondata, 1);
-                     }
-                     if (drawgap) {
-                        // draw dead pixels at bottom edge of cell
-                        for (int j = 0; j <= cellsize; j++) {
-                           p.Red()   = deadred;
-                           p.Green() = deadgreen;
-                           p.Blue()  = deadblue;
-                           #ifdef __WXGTK__
-                              p.Alpha() = 255;
-                           #endif
-                           p++;
-                        }
-                     }
-                  }
-               } else {
-                  // draw dead cell
                   for (int i = 0; i < cellsize; i++) {
                      wxAlphaPixelData::Iterator colstart = p;
+                     #ifdef __WXMSW__
+                        wxNativePixelData::Iterator iconrow = iconpxl;
+                     #else
+                        wxAlphaPixelData::Iterator iconrow = iconpxl;
+                     #endif
                      for (int j = 0; j < cellsize; j++) {
-                        p.Red()   = deadred;
-                        p.Green() = deadgreen;
-                        p.Blue()  = deadblue;
-                        #ifdef __WXGTK__
+                        if (iconpxl.Red() || iconpxl.Green() || iconpxl.Blue()) {
+                           if (multicolor) {
+                              // use non-black pixel in multi-colored icon
+                              if (swapcolors) {
+                                 p.Red()   = 255 - iconpxl.Red();
+                                 p.Green() = 255 - iconpxl.Green();
+                                 p.Blue()  = 255 - iconpxl.Blue();
+                              } else {
+                                 p.Red()   = iconpxl.Red();
+                                 p.Green() = iconpxl.Green();
+                                 p.Blue()  = iconpxl.Blue();
+                              }
+                           } else {
+                              // replace non-black pixel with current cell color
+                              p.Red()   = currlayer->cellr[state];
+                              p.Green() = currlayer->cellg[state];
+                              p.Blue()  = currlayer->cellb[state];
+                           }
+                        } else {
+                           // replace black pixel with dead cell color
+                           p.Red()   = deadred;
+                           p.Green() = deadgreen;
+                           p.Blue()  = deadblue;
+                        }
+                        #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
                            p.Alpha() = 255;
                         #endif
                         p++;
+                        iconpxl++;
                      }
                      if (drawgap) {
                         // draw dead pixels at right edge of cell
                         p.Red()   = deadred;
                         p.Green() = deadgreen;
                         p.Blue()  = deadblue;
-                        #ifdef __WXGTK__
+                        #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
                            p.Alpha() = 255;
                         #endif
                      }
                      p = colstart;
                      p.OffsetY(pxldata, 1);
+                     // move to next row of icon bitmap
+                     iconpxl = iconrow;
+                     iconpxl.OffsetY(icondata, 1);
                   }
                   if (drawgap) {
                      // draw dead pixels at bottom edge of cell
@@ -672,19 +658,53 @@ void DrawIcons(unsigned char* byteptr, int x, int y, int w, int h, int pmscale)
                         p.Red()   = deadred;
                         p.Green() = deadgreen;
                         p.Blue()  = deadblue;
-                        #ifdef __WXGTK__
+                        #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
                            p.Alpha() = 255;
                         #endif
                         p++;
                      }
                   }
                }
-               p = topleft;
-               
+            } else {
+               // draw dead cell
+               for (int i = 0; i < cellsize; i++) {
+                  wxAlphaPixelData::Iterator colstart = p;
+                  for (int j = 0; j < cellsize; j++) {
+                     p.Red()   = deadred;
+                     p.Green() = deadgreen;
+                     p.Blue()  = deadblue;
+                     #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
+                        p.Alpha() = 255;
+                     #endif
+                     p++;
+                  }
+                  if (drawgap) {
+                     // draw dead pixels at right edge of cell
+                     p.Red()   = deadred;
+                     p.Green() = deadgreen;
+                     p.Blue()  = deadblue;
+                     #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
+                        p.Alpha() = 255;
+                     #endif
+                  }
+                  p = colstart;
+                  p.OffsetY(pxldata, 1);
+               }
+               if (drawgap) {
+                  // draw dead pixels at bottom edge of cell
+                  for (int j = 0; j <= cellsize; j++) {
+                     p.Red()   = deadred;
+                     p.Green() = deadgreen;
+                     p.Blue()  = deadblue;
+                     #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
+                        p.Alpha() = 255;
+                     #endif
+                     p++;
+                  }
+               }
             }
+            p = topleft;
             p.OffsetX(pxldata, pmscale);
-            
-            byteptr++;        // move to next byte in pmdata
          }
          p = rowstart;
          p.OffsetY(pxldata, pmscale);
@@ -708,7 +728,7 @@ void DrawOneIcon(wxDC& dc, int x, int y, wxBitmap* icon,
 
    wxAlphaPixelData pxldata(pixmap);
    if (pxldata) {
-      #ifdef __WXGTK__
+      #if defined(__WXGTK__) && !wxCHECK_VERSION(2,9,0)
          pxldata.UseAlpha();
       #endif
       wxAlphaPixelData::Iterator p(pxldata);
@@ -759,7 +779,7 @@ void DrawOneIcon(wxDC& dc, int x, int y, wxBitmap* icon,
                   p.Green() = deadg;
                   p.Blue()  = deadb;
                }
-               #ifdef __WXGTK__
+               #if defined(__WXGTK__) || wxCHECK_VERSION(2,9,0)
                   p.Alpha() = 255;
                #endif
                p++;
@@ -831,6 +851,46 @@ void wx_render::pixblit(int x, int y, int w, int h, char* pmdata, int pmscale)
    if (x >= currwd || y >= currht) return;
    if (x + w <= 0 || y + h <= 0) return;
 
+   // stride is the horizontal pixel width of the image data
+   int stride = (pmscale == 1) ? w : w/pmscale;
+
+   // clip pixmap to be drawn against viewport:
+   if (pmscale == 1)
+   {
+      // pmdata contains 3 bytes per pixel
+      if (x < 0) {
+         pmdata -= 3*x;
+         w += x;
+         x = 0;
+      }
+      if (y < 0) {
+         pmdata -= 3*y*stride;
+         h += y;
+         y = 0;
+      }
+      if (w > currwd) w = currwd;
+      if (h > currht) h = currht;
+   }
+   else
+   {
+      // pmdata contains 1 byte per `pmscale' pixels, so we must be careful
+      // and adjust x, y, w and h by multiples of `pmscale' only.
+      if (x < 0) {
+         int dx = -x/pmscale*pmscale;
+         pmdata += dx/pmscale;
+         w -= dx;
+         x += dx;
+      }
+      if (y < 0) {
+         int dy = -y/pmscale*pmscale;
+         pmdata += dy/pmscale*stride;
+         h -= dy;
+         y += dy;
+      }
+      if (x + w >= currwd + pmscale) w = (currwd - x + pmscale - 1)/pmscale*pmscale;
+      if (y + h >= currht + pmscale) h = (currht - y + pmscale - 1)/pmscale*pmscale;
+   }
+
    // faster to create new pixmap only when size changes
    if (pixmapwd != w || pixmapht != h) {
       delete pixmap;
@@ -840,39 +900,17 @@ void wx_render::pixblit(int x, int y, int w, int h, char* pmdata, int pmscale)
    }
 
    if (pmscale == 1) {
-      // pmdata contains 3 bytes (the rgb values) for each pixel
-      wxAlphaPixelData pxldata(*pixmap);
-      if (pxldata) {
-         #ifdef __WXGTK__
-            pxldata.UseAlpha();
-         #endif
-         wxAlphaPixelData::Iterator p(pxldata);
-         unsigned char* byteptr = (unsigned char*) pmdata;
-         for ( int row = 0; row < h; row++ ) {
-            wxAlphaPixelData::Iterator rowstart = p;
-            for ( int col = 0; col < w; col++ ) {
-               p.Red()   = *byteptr++;
-               p.Green() = *byteptr++;
-               p.Blue()  = *byteptr++;
-               #ifdef __WXGTK__
-                  p.Alpha() = 255;
-               #endif
-               p++;
-            }
-            p = rowstart;
-            p.OffsetY(pxldata, 1);
-         }
-      }
-      currdc->DrawBitmap(*pixmap, x, y);
+      // draw rgb pixel data
+      DrawPixmap((unsigned char*) pmdata, x, y, w, h, stride);
 
    } else if (showicons && pmscale > 4 && iconmaps) {
       // draw icons only at scales 1:8 or 1:16
-      DrawIcons((unsigned char*) pmdata, x, y, w/pmscale, h/pmscale, pmscale);
+      DrawIcons((unsigned char*) pmdata, x, y, w/pmscale, h/pmscale, pmscale, stride);
 
    } else {
       // stretch pixmap by pmscale, assuming pmdata contains (w/pmscale)*(h/pmscale) bytes
       // where each byte contains a cell state
-      DrawStretchedPixmap((unsigned char*) pmdata, x, y, w/pmscale, h/pmscale, pmscale);
+      DrawStretchedPixmap((unsigned char*) pmdata, x, y, w/pmscale, h/pmscale, pmscale, stride);
    }
 }
 
@@ -929,6 +967,7 @@ void DrawSelection(wxDC& dc, wxRect& rect)
          wxMemoryDC memdc;
          memdc.SelectObject(*selbitmap);
          dc.Blit(rect.x, rect.y, rect.width, rect.height, &memdc, 0, 0, wxCOPY, true);
+         memdc.SelectObject(wxNullBitmap);
       #endif
    } else {
       // no alpha channel so just invert rect
@@ -954,6 +993,7 @@ void DrawInactiveSelection(wxDC& dc, wxRect& rect)
          wxMemoryDC memdc;
          memdc.SelectObject(*graybitmap);
          dc.Blit(rect.x, rect.y, rect.width, rect.height, &memdc, 0, 0, wxCOPY, true);
+         memdc.SelectObject(wxNullBitmap);
       #endif
    } else {
       // no alpha channel so just invert rect
@@ -999,7 +1039,9 @@ void MaskDeadPixels(wxBitmap* bitmap, int wd, int ht, int livealpha)
       int deadg = currlayer->cellg[0];
       int deadb = currlayer->cellb[0];
 
-      data.UseAlpha();
+      #if !wxCHECK_VERSION(2,9,0)
+         data.UseAlpha();
+      #endif
       wxAlphaPixelData::Iterator p(data);
       for ( int y = 0; y < ht; y++ ) {
          wxAlphaPixelData::Iterator rowstart = p;
@@ -1018,8 +1060,8 @@ void MaskDeadPixels(wxBitmap* bitmap, int wd, int ht, int livealpha)
                p.Alpha() = 0;
             } else {
                // live pixel
-               #ifdef __WXMSW__
-                  // premultiply the RGB values on Windows
+               #if defined(__WXMSW__) || (defined(__WXMAC__) && wxCHECK_VERSION(2,8,8))
+                  // premultiply the RGB values
                   p.Red()   = r * livealpha / 255;
                   p.Green() = g * livealpha / 255;
                   p.Blue()  = b * livealpha / 255;
@@ -1197,11 +1239,11 @@ void CheckPasteImage()
          
          wxMemoryDC pattdc;
          pattdc.SelectObject(*pastebitmap);
-         
          currdc = &pattdc;
          currwd = tempview.getwidth();
          currht = tempview.getheight();
          pastealgo->draw(tempview, renderer);
+         pattdc.SelectObject(wxNullBitmap);
          
          showgridlines = saveshow;
          
@@ -1254,6 +1296,7 @@ void DrawPasteImage(wxDC& dc)
          wxMemoryDC memdc;
          memdc.SelectObject(*pastebitmap);
          dc.Blit(r.x, r.y, pimagewd, pimageht, &memdc, 0, 0, wxCOPY, true);
+         memdc.SelectObject(wxNullBitmap);
       #endif
    }
 
@@ -1466,6 +1509,7 @@ void DrawOneLayer(wxDC& dc)
    
    currdc = &layerdc;
    currlayer->algo->draw(*currlayer->view, renderer);
+   layerdc.SelectObject(wxNullBitmap);
    
    // make dead pixels 100% transparent; live pixels use opacity setting
    MaskDeadPixels(layerbitmap, layerwd, layerht, int(2.55 * opacity));
