@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -46,13 +46,6 @@ bool ValidRule(wxString& rule)
    // return true if given rule is valid in at least one algorithm
    // and convert rule to canonical form
    
-   wxString oldrule = wxEmptyString;
-   if (currlayer->algtype == QLIFE_ALGO || currlayer->algtype == HLIFE_ALGO) {
-      // qlife and hlife share global_liferules, so we need to save
-      // and restore the current rule -- yuk
-      oldrule = wxString(currlayer->algo->getrule(),wxConvLocal);
-   }
-   
    for (int i = 0; i < NumAlgos(); i++) {
       lifealgo* tempalgo = CreateNewUniverse(i);
       const char* err = tempalgo->setrule( rule.mb_str(wxConvLocal) );
@@ -60,13 +53,11 @@ bool ValidRule(wxString& rule)
          // convert rule to canonical form
          rule = wxString(tempalgo->getrule(),wxConvLocal);
          delete tempalgo;
-         if (!oldrule.IsEmpty()) currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
          return true;
       }
       delete tempalgo;
    }
    
-   if (!oldrule.IsEmpty()) currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
    return false;
 }
 
@@ -222,7 +213,7 @@ void AlgoHelp::OnLinkClicked(const wxHtmlLinkInfo& link)
          wxWindow* buttwin = GetParent()->FindWindow(wxID_OK);
          if (buttwin) {
             okevent.SetEventObject(buttwin);
-            buttwin->ProcessEvent(okevent);
+            buttwin->GetEventHandler()->ProcessEvent(okevent);
          }
       }
 
@@ -280,6 +271,12 @@ void AlgoHelp::ClearStatus()
 
 // -----------------------------------------------------------------------------
 
+#if defined(__WXMAC__) && wxCHECK_VERSION(2,9,0)
+   // wxMOD_CONTROL has been changed to mean Command key down (sheesh!)
+   #define wxMOD_CONTROL wxMOD_RAW_CONTROL
+   #define ControlDown RawControlDown
+#endif
+
 void AlgoHelp::OnMouseDown(wxMouseEvent& event)
 {
    // set flag so ctrl/right-clicked file can be opened in editor
@@ -396,7 +393,11 @@ void AlgoHelp::SetFontSizes(int size)
    f_sizes[4] = int(size * 1.4);
    f_sizes[5] = int(size * 1.6);
    f_sizes[6] = int(size * 1.8);
-   SetFonts(wxEmptyString, wxEmptyString, f_sizes);
+   #ifdef __WXOSX_COCOA__
+      SetFonts(wxT("Lucida Grande"), wxT("Monaco"), f_sizes);
+   #else
+      SetFonts(wxEmptyString, wxEmptyString, f_sizes);
+   #endif
 }
 
 // -----------------------------------------------------------------------------
@@ -407,8 +408,8 @@ const wxString UNNAMED = _("UNNAMED");    // unnamed rule
 const int HGAP = 12;
 const int BIGVGAP = 12;
 
-#if defined(__WXMAC__) && wxCHECK_VERSION(2,8,0)
-   // fix wxALIGN_CENTER_VERTICAL bug in wxMac 2.8.0+;
+#if defined(__WXMAC__) && wxCHECK_VERSION(2,8,0) && !wxCHECK_VERSION(2,9,0)
+   // fix wxALIGN_CENTER_VERTICAL bug in wxMac 2.8.x;
    // only happens when a wxStaticText/wxButton box is next to a wxChoice box
    #define FIX_ALIGN_BUG wxBOTTOM,4
 #else
@@ -526,7 +527,7 @@ RuleDialog::RuleDialog(wxWindow* parent)
       wxWindow* buttwin = FindWindow(wxID_HELP);
       if (buttwin) {
          buttevent.SetEventObject(buttwin);
-         buttwin->ProcessEvent(buttevent);
+         buttwin->GetEventHandler()->ProcessEvent(buttevent);
       }
    }
 }
@@ -1039,17 +1040,21 @@ bool RuleDialog::TransferDataFromWindow()
    }
    
    if (algoindex == currlayer->algtype) {
-      // new rule should be valid in current algorithm
+      // check if new rule is still valid in current algorithm
       const char* err = currlayer->algo->setrule( newrule.mb_str(wxConvLocal) );
-      if (!err) {
-         // switch to default colors for new rule
-         UpdateLayerColors();
+      if (err) {
+         // this can happen if the rule's table/tree file was deleted,
+         // or was edited and some sort of error introduced
+         Warning(_("This rule is no longer valid!"));
+         ruletext->SetFocus();
+         ruletext->SetSelection(-1,-1);
+         return false;
+      } else {
          return true;
       }
-      Warning(_("Bug detected in TransferDataFromWindow!"));
-      return false;
    } else {
       // change the current algorithm and switch to the new rule
+      // (if the new rule is invalid then the algo's default rule will be used)
       mainptr->ChangeAlgorithm(algoindex, newrule);
       return true;
    }
@@ -1068,10 +1073,11 @@ bool ChangeRule()
       // and possibly the current algorithm as well
       return true;
    } else {
-      // user hit Cancel so restore rule and name array
-      currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+      // user hit Cancel so restore name array and rule
       namedrules.Clear();
       namedrules = oldnames;
-      return false;
+      return !RestoreRule(oldrule);
+      // note that we return true if RestoreRule failed and had to
+      // switch to the current algorithm's default rule
    }
 }
