@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -51,20 +51,24 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 // exported globals:
 bool inscript = false;     // a script is running?
+bool passkeys;             // pass keyboard events to script?
+bool passclicks;           // pass mouse events to script?
 bool canswitch;            // can user switch layers while script is running?
 bool stop_after_script;    // stop generating pattern after running script?
 bool autoupdate;           // update display after each change to current universe?
 bool allowcheck;           // allow event checking?
 wxString scripterr;        // Perl/Python error message
+wxString mousepos;         // current mouse position
 
 // local globals:
-static bool plscript = false;     // a Perl script is running?
-static bool pyscript = false;     // a Python script is running?
-static bool showtitle;            // need to update window title?
-static bool updateedit;           // need to update edit bar?
-static bool exitcalled;           // GSF_exit was called?
-static wxString scriptchars;      // non-escape chars saved by PassKeyToScript
-static wxString scriptloc;        // location of script file
+static bool plscript = false;       // a Perl script is running?
+static bool pyscript = false;       // a Python script is running?
+static bool showtitle;              // need to update window title?
+static bool updateedit;             // need to update edit bar?
+static bool exitcalled;             // GSF_exit was called?
+static wxString scriptchars;        // non-escape chars saved by PassKeyToScript
+static wxString scriptloc;          // location of script file
+static wxArrayString eventqueue;    // FIFO queue for keyboard/mouse events 
 
 // -----------------------------------------------------------------------------
 
@@ -225,11 +229,7 @@ const char* GSF_setalgo(char* algostring)
          break;
       }
    }
-   if (algoindex < 0) {
-      static char err[256];
-      sprintf(err, "Unknown algorithm: %s", algostring);
-      return err;
-   }
+   if (algoindex < 0) return "Unknown algorithm.";
    
    if (algoindex != currlayer->algtype) {
       mainptr->ChangeAlgorithm(algoindex);
@@ -282,7 +282,7 @@ const char* GSF_setrule(char* rulestring)
                // change the current algorithm and switch to the new rule
                mainptr->ChangeAlgorithm(i, wxString(rulestring,wxConvLocal));
                if (i != currlayer->algtype) {
-                  currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+                  RestoreRule(oldrule);
                   return "Algorithm could not be changed (pattern is too big to convert).";
                } else {
                   ChangeWindowTitle(wxEmptyString);
@@ -292,12 +292,15 @@ const char* GSF_setrule(char* rulestring)
             }
          }
       }
-      currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+      RestoreRule(oldrule);
       return "Given rule is not valid in any algorithm.";
    }
 
+   // check if the rule string changed, or the number of states changed
+   // (the latter might happen if user edited a table/tree file)
    wxString newrule = wxString(currlayer->algo->getrule(),wxConvLocal);
-   if (oldrule != newrule) {
+   int newmaxstate = currlayer->algo->NumCellStates() - 1;
+   if (oldrule != newrule || oldmaxstate != newmaxstate) {
       // show new rule in main window's title but don't change name
       ChangeWindowTitle(wxEmptyString);
 
@@ -308,7 +311,6 @@ const char* GSF_setrule(char* rulestring)
 
       // rule change might have changed the number of cell states;
       // if there are fewer states then pattern might change
-      int newmaxstate = currlayer->algo->NumCellStates() - 1;
       if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
          mainptr->ReduceCellStates(newmaxstate);
       }
@@ -533,47 +535,28 @@ int GSF_hash(int x, int y, int wd, int ht)
    int bottom = y + ht - 1;
    int cx, cy;
    int v = 0;
-
    lifealgo* curralgo = currlayer->algo;
-   if (curralgo->NumCellStates() > 2) {
-      // multi-state universe
-      for ( cy=y; cy<=bottom; cy++ ) {
-         int yshift = cy - y;
-         for ( cx=x; cx<=right; cx++ ) {
-            int skip = curralgo->nextcell(cx, cy, v);
-            if (skip >= 0) {
-               // found next live cell in this row (v is >= 1)
-               cx += skip;
-               if (cx <= right) {
-                  // need to use a good hash function for patterns like AlienCounter.rle
-                  hash = (hash * 1000003) ^ yshift;
-                  hash = (hash * 1000003) ^ (cx - x);
-                  hash = (hash * 1000003) ^ v;
-               }
-            } else {
-               cx = right;  // done this row
-            }
-         }
-      }
-   } else {
-      // two-state universe
-      for ( cy=y; cy<=bottom; cy++ ) {
-         int yshift = cy - y;
-         for ( cx=x; cx<=right; cx++ ) {
-            int skip = curralgo->nextcell(cx, cy, v);
-            if (skip >= 0) {
-               // found next live cell in this row (v is 1)
-               cx += skip;
-               if (cx <= right) {
-                  hash = (hash * 33 + yshift) ^ (cx - x);
-               }
-            } else {
-               cx = right;  // done this row
+   bool multistate = curralgo->NumCellStates() > 2;
+   
+   for ( cy=y; cy<=bottom; cy++ ) {
+      int yshift = cy - y;
+      for ( cx=x; cx<=right; cx++ ) {
+         int skip = curralgo->nextcell(cx, cy, v);
+         if (skip >= 0) {
+            // found next live cell in this row (v is >= 1 if multistate)
+            cx += skip;
+            if (cx <= right) {
+               // need to use a good hash function for patterns like AlienCounter.rle
+               hash = (hash * 1000003) ^ yshift;
+               hash = (hash * 1000003) ^ (cx - x);
+               if (multistate) hash = (hash * 1000003) ^ v;
             }
+         } else {
+            cx = right;  // done this row
          }
       }
    }
-
+   
    return hash;
 }
 
@@ -1003,9 +986,174 @@ bool GSF_getcolor(char* colname, wxColor& color)
 
 // -----------------------------------------------------------------------------
 
+void GSF_getevent(wxString& event, int get)
+{
+   if (get) {
+      passkeys = true;     // future keyboard events will call PassKeyToScript
+      passclicks = true;   // future mouse events will call PassClickToScript
+   } else {
+      // tell Golly to handle future keyboard and mouse events
+      passkeys = false;
+      passclicks = false;
+      // clear any pending events so event is set to empty string below
+      eventqueue.Clear();
+   }
+   
+   if (eventqueue.IsEmpty()) {
+      event = wxEmptyString;
+   } else {
+      // get event at start of queue, then remove it
+      event = eventqueue[0];
+      eventqueue.RemoveAt(0);
+   }
+}
+
+// -----------------------------------------------------------------------------
+
+#if defined(__WXMAC__) && wxCHECK_VERSION(2,9,0)
+   // wxMOD_CONTROL has been changed to mean Command key down (sheesh!)
+   #define wxMOD_CONTROL wxMOD_RAW_CONTROL
+   #define ControlDown RawControlDown
+#endif
+
+static void AppendModifiers(int modifiers, wxString& event)
+{
+   // reverse of GetModifiers
+   if (modifiers == wxMOD_NONE) {
+      event += wxT("none");
+   } else {
+      if (modifiers & wxMOD_ALT)       event += wxT("alt");
+      #ifdef __WXMAC__
+         if (modifiers & wxMOD_CMD)       event += wxT("cmd");
+         if (modifiers & wxMOD_CONTROL)   event += wxT("ctrl");
+      #else
+         if (modifiers & wxMOD_CMD)       event += wxT("ctrl");
+         if (modifiers & wxMOD_META)      event += wxT("meta");
+      #endif
+      if (modifiers & wxMOD_SHIFT)     event += wxT("shift");
+   }
+}
+
+// -----------------------------------------------------------------------------
+
+static int GetModifiers(const wxString& modstring)
+{
+   // reverse of AppendModifiers
+   int modifiers = wxMOD_NONE;
+   if (modstring != wxT("none")) {
+      if (modstring.Contains(wxT("alt")))     modifiers |= wxMOD_ALT;
+      if (modstring.Contains(wxT("cmd")))     modifiers |= wxMOD_CMD;
+      if (modstring.Contains(wxT("ctrl")))    modifiers |= wxMOD_CONTROL;
+      if (modstring.Contains(wxT("meta")))    modifiers |= wxMOD_META;
+      if (modstring.Contains(wxT("shift")))   modifiers |= wxMOD_SHIFT;
+   }
+   return modifiers;
+}
+
+// -----------------------------------------------------------------------------
+
+const char* GSF_doevent(const wxString& event)
+{
+   if (event.length() > 0) {
+      if (event.StartsWith(wxT("key ")) && event.length() > 7) {
+         // parse event string like "key x altshift"
+         int key = event[4];
+         if (event[4] == 'f' && event[5] >= '1' && event[5] <= '9') {
+            // parse function key (f1 to f24)
+            if (event[6] == ' ') {
+               // f1 to f9
+               key = WXK_F1 + (event[5] - '1');
+            } else if (event[6] >= '0' && event[6] <= '9') {
+               // f10 to f24
+               key = WXK_F1 + 10 * (event[5] - '0') + (event[6] - '0') - 1;
+               if (key > WXK_F24)
+                  return "Bad function key (must be f1 to f24).";
+            } else {
+               return "Bad function key (must be f1 to f24).";
+            }
+         } else if (event[5] != ' ') {
+            // parse special char name like space, tab, etc
+            // must match reverse conversion in PassKeyToScript
+            if (event.Contains(wxT("space")))      key = ' '; else
+            if (event.Contains(wxT("home")))       key = WXK_HOME; else
+            if (event.Contains(wxT("end")))        key = WXK_END; else
+            if (event.Contains(wxT("pageup")))     key = WXK_PAGEUP; else
+            if (event.Contains(wxT("pagedown")))   key = WXK_PAGEDOWN; else
+            if (event.Contains(wxT("help")))       key = WXK_HELP; else
+            if (event.Contains(wxT("insert")))     key = WXK_INSERT; else
+            if (event.Contains(wxT("delete")))     key = WXK_DELETE; else
+            if (event.Contains(wxT("tab")))        key = WXK_TAB; else
+            if (event.Contains(wxT("enter")))      key = WXK_RETURN; else
+            if (event.Contains(wxT("return")))     key = WXK_RETURN; else
+            if (event.Contains(wxT("left")))       key = WXK_LEFT; else
+            if (event.Contains(wxT("right")))      key = WXK_RIGHT; else
+            if (event.Contains(wxT("up")))         key = WXK_UP; else
+            if (event.Contains(wxT("down")))       key = WXK_DOWN; else
+            return "Unknown key.";
+         }
+
+         viewptr->ProcessKey(key, GetModifiers(event.AfterLast(' ')));
+   
+         if (showtitle) {
+            // update window title
+            inscript = false;
+            mainptr->SetWindowTitle(wxEmptyString);
+            inscript = true;
+            showtitle = false;
+         }
+         
+      } else if (event.StartsWith(wxT("click "))) {
+         // parse event string like "click 10 20 left altshift"
+         wxString xstr = event.AfterFirst(' ');
+         wxString ystr = xstr.AfterFirst(' ');
+         xstr = xstr.BeforeFirst(' ');
+         ystr = ystr.BeforeFirst(' ');
+         if (!xstr.IsNumber()) return "Bad x value.";
+         if (!ystr.IsNumber()) return "Bad y value.";
+         bigint x(xstr.mb_str(wxConvLocal));
+         bigint y(ystr.mb_str(wxConvLocal));
+         
+         int button;
+         if (event.Contains(wxT(" left "))) button = wxMOUSE_BTN_LEFT; else
+         if (event.Contains(wxT(" middle "))) button = wxMOUSE_BTN_MIDDLE; else
+         if (event.Contains(wxT(" right "))) button = wxMOUSE_BTN_RIGHT; else
+         return "Unknown button.";
+
+         if (viewptr->CellVisible(x, y) && viewptr->CellInGrid(x, y)) {
+            // convert x,y cell position to pixel position in viewport
+            pair<int,int> xy = currlayer->view->screenPosOf(x, y, currlayer->algo);
+            int mods = GetModifiers(event.AfterLast(' '));
+            
+            viewptr->ProcessClick(xy.first, xy.second, button, mods);
+      
+            if (showtitle) {
+               // update window title
+               inscript = false;
+               mainptr->SetWindowTitle(wxEmptyString);
+               inscript = true;
+               showtitle = false;
+            }
+         } else {
+            // ignore click if x,y is outside viewport or grid
+            return NULL;
+         }
+         
+      } else {
+         return "Unknown event.";
+      }
+   }
+   return NULL;
+}
+
+// -----------------------------------------------------------------------------
+
+// the following is deprecated (use GSF_getevent)
+
 void GSF_getkey(char* s)
 {
-   if (scriptchars.Length() == 0) {
+   passkeys = true;   // future keyboard events will call PassKeyToScript
+   
+   if (scriptchars.length() == 0) {
       // return empty string
       s[0] = '\0';
    } else {
@@ -1018,10 +1166,7 @@ void GSF_getkey(char* s)
 
 // -----------------------------------------------------------------------------
 
-// also allow mouse interaction??? ie. g.doclick( g.getclick() ) where
-// getclick returns [] or [x,y,button,modifiers];
-// or maybe it would be better to implement Perl/Python callback routines
-// like OnKey and OnClick that a script can override???
+// the following is deprecated (use GSF_doevent)
 
 void GSF_dokey(char* ascii)
 {
@@ -1042,22 +1187,16 @@ void GSF_dokey(char* ascii)
          default:    key = *ascii;
       }
 
-      // can we handle modifiers and be backward compatible???
-      // or do we need new getmodkey & domodkey commands???
+      // we can't handle modifiers here
       viewptr->ProcessKey(key, wxMOD_NONE);
 
-      // see any cursor change, including in edit bar
-      mainptr->UpdateUserInterface(mainptr->IsActive());
-
-      // update viewport, status bar, scroll bars, etc
-      inscript = false;
-      mainptr->UpdatePatternAndStatus();
-      bigview->UpdateScrollBars();
       if (showtitle) {
+         // update window title
+         inscript = false;
          mainptr->SetWindowTitle(wxEmptyString);
+         inscript = true;
          showtitle = false;
       }
-      inscript = true;
    }
 }
 
@@ -1193,6 +1332,7 @@ void RunScript(const wxString& filename)
       statusptr->ClearMessage();
       scripterr.Clear();
       scriptchars.Clear();
+      eventqueue.Clear();
       canswitch = false;
       stop_after_script = false;
       autoupdate = false;
@@ -1200,6 +1340,8 @@ void RunScript(const wxString& filename)
       allowcheck = true;
       showtitle = false;
       updateedit = false;
+      passkeys = false;
+      passclicks = false;
       wxGetApp().PollerReset();
    }
 
@@ -1238,9 +1380,8 @@ void RunScript(const wxString& filename)
    
       mainptr->UpdateUserInterface(mainptr->IsActive());
 
-      // temporarily clear non-ctrl and non-func key accelerators from
-      // menu items so keys like tab/enter/space can be passed to script
-      // even if menu item is disabled
+      // temporarily remove accelerators from all menu items
+      // so keyboard shortcuts can be passed to script
       mainptr->UpdateMenuAccelerators();
    }
 
@@ -1342,7 +1483,24 @@ void RunScript(const wxString& filename)
 
 // -----------------------------------------------------------------------------
 
-void PassKeyToScript(int key)
+void PassClickToScript(const bigint& x, const bigint& y, int button, int modifiers)
+{
+   // build a string like "click 10 20 left altshift" and add to event queue
+   // for possible consumption by GSF_getevent
+   wxString clickinfo = wxT("click ");
+   clickinfo += wxString(x.tostring('\0'),wxConvLocal);
+   clickinfo += wxT(" ");
+   clickinfo += wxString(y.tostring('\0'),wxConvLocal);
+   if (button == wxMOUSE_BTN_LEFT)     clickinfo += wxT(" left ");
+   if (button == wxMOUSE_BTN_MIDDLE)   clickinfo += wxT(" middle ");
+   if (button == wxMOUSE_BTN_RIGHT)    clickinfo += wxT(" right ");
+   AppendModifiers(modifiers, clickinfo);
+   eventqueue.Add(clickinfo);
+}
+
+// -----------------------------------------------------------------------------
+
+void PassKeyToScript(int key, int modifiers)
 {
    if (key == WXK_ESCAPE) {
       if (mainptr->generating) {
@@ -1352,6 +1510,48 @@ void PassKeyToScript(int key)
       if (plscript) AbortPerlScript();
       if (pyscript) AbortPythonScript();
    } else {
+      // build a string like "key x altshift" and add to event queue
+      // for possible consumption by GSF_getevent
+      wxString keyinfo = wxT("key ");
+      if (key > ' ' && key <= '~') {
+         // displayable ASCII
+         keyinfo += wxChar(key);
+      } else if (key >= WXK_F1 && key <= WXK_F24) {
+         // function key
+         keyinfo += wxString::Format(wxT("f%d"), key - WXK_F1 + 1);
+      } else {
+         // convert some special key codes to names like space, tab, delete, etc
+         // (must match reverse conversion in GSF_doevent)
+         switch (key) {
+            case ' ':               keyinfo += wxT("space");      break;
+            case WXK_HOME:          keyinfo += wxT("home");       break;
+            case WXK_END:           keyinfo += wxT("end");        break;
+            case WXK_PAGEUP:        keyinfo += wxT("pageup");     break;
+            case WXK_PAGEDOWN:      keyinfo += wxT("pagedown");   break;
+            case WXK_HELP:          keyinfo += wxT("help");       break;
+            case WXK_INSERT:        keyinfo += wxT("insert");     break;
+            case WXK_BACK:          // treat backspace like delete
+            case WXK_DELETE:        keyinfo += wxT("delete");     break;
+            case WXK_TAB:           keyinfo += wxT("tab");        break;
+            case WXK_NUMPAD_ENTER:  // treat enter like return
+            case WXK_RETURN:        keyinfo += wxT("return");     break;
+            case WXK_LEFT:          keyinfo += wxT("left");       break;
+            case WXK_RIGHT:         keyinfo += wxT("right");      break;
+            case WXK_UP:            keyinfo += wxT("up");         break;
+            case WXK_DOWN:          keyinfo += wxT("down");       break;
+            case WXK_ADD:           keyinfo += wxT("+");          break;
+            case WXK_SUBTRACT:      keyinfo += wxT("-");          break;
+            case WXK_DIVIDE:        keyinfo += wxT("/");          break;
+            case WXK_MULTIPLY:      keyinfo += wxT("*");          break;
+            default:                return;  // ignore all other key codes
+         }
+      }
+      keyinfo += wxT(" ");
+      AppendModifiers(modifiers, keyinfo);
+      eventqueue.Add(keyinfo);
+   
+      // NOTE: following code is for deprecated getkey() command
+      
       // convert wx key code to corresponding ascii char (if possible)
       // so that scripts can be platform-independent;
       // note that GSF_dokey does the reverse conversion
@@ -1359,29 +1559,30 @@ void PassKeyToScript(int key)
       Warning(wxString::Format(_("key=%d ch=%c"), key, (char)key));
       */
       char ascii;
-      switch (key) {
-         case WXK_DELETE:     // treat delete like backspace
-         case WXK_BACK:       ascii = 8;     break;
-         case WXK_TAB:        ascii = 9;     break;
-         case WXK_NUMPAD_ENTER: // treat enter like return
-         case WXK_RETURN:     ascii = 13;    break;
-         case WXK_LEFT:       ascii = 28;    break;
-         case WXK_RIGHT:      ascii = 29;    break;
-         case WXK_UP:         ascii = 30;    break;
-         case WXK_DOWN:       ascii = 31;    break;
-         case WXK_ADD:        ascii = '+';   break;
-         case WXK_SUBTRACT:   ascii = '-';   break;
-         case WXK_DIVIDE:     ascii = '/';   break;
-         case WXK_MULTIPLY:   ascii = '*';   break;
-         // map F1..F12 to ascii 14..25???
-         default:
-            if (key >= ' ' && key <= '~') {
-               ascii = key;
-            } else {
-               // ignore all other key codes???
-               // or maybe allow codes < ' ' but ignore > '~'???
-               return;
-            }
+      if (key >= ' ' && key <= '~') {
+         if (modifiers == wxMOD_SHIFT && key >= 'a' && key <= 'z') {
+            // let script see A..Z
+            ascii = key - 32;
+         } else {
+            ascii = key;
+         }
+      } else {
+         switch (key) {
+            case WXK_DELETE:     // treat delete like backspace
+            case WXK_BACK:       ascii = 8;     break;
+            case WXK_TAB:        ascii = 9;     break;
+            case WXK_NUMPAD_ENTER: // treat enter like return
+            case WXK_RETURN:     ascii = 13;    break;
+            case WXK_LEFT:       ascii = 28;    break;
+            case WXK_RIGHT:      ascii = 29;    break;
+            case WXK_UP:         ascii = 30;    break;
+            case WXK_DOWN:       ascii = 31;    break;
+            case WXK_ADD:        ascii = '+';   break;
+            case WXK_SUBTRACT:   ascii = '-';   break;
+            case WXK_DIVIDE:     ascii = '/';   break;
+            case WXK_MULTIPLY:   ascii = '*';   break;
+            default:             return;  // ignore all other key codes
+         }
       }
       // save ascii char for possible consumption by GSF_getkey
       scriptchars += ascii;
