@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -154,13 +154,13 @@ void StatusBar::UpdateXYLocation()
 
 void StatusBar::CheckMouseLocation(bool active)
 {
-   if (statusht == 0)
-      return;
+   if (statusht == 0 && !inscript) return;
 
    if ( !active ) {
       // main window is not in front so clear XY location
       showxy = false;
-      UpdateXYLocation();
+      if (statusht > 0) UpdateXYLocation();
+      if (inscript) mousepos = wxEmptyString;
       return;
    }
 
@@ -172,15 +172,21 @@ void StatusBar::CheckMouseLocation(bool active)
          currx = xpos;
          curry = ypos;
          showxy = true;
-         UpdateXYLocation();
+         if (statusht > 0) UpdateXYLocation();
       } else if (!showxy) {
          showxy = true;
-         UpdateXYLocation();
+         if (statusht > 0) UpdateXYLocation();
+      }
+      if (inscript) {
+         mousepos = wxString(xpos.tostring('\0'), wxConvLocal);
+         mousepos += wxT(" ");
+         mousepos += wxString(ypos.tostring('\0'), wxConvLocal);
       }
    } else {
       // outside viewport so clear XY location
       showxy = false;
-      UpdateXYLocation();
+      if (statusht > 0) UpdateXYLocation();
+      if (inscript) mousepos = wxEmptyString;
    }
 }
 
@@ -399,7 +405,7 @@ void StatusBar::DrawStatusBar(wxDC& dc, wxRect& updaterect)
             ypos = temp;
          }
          strbuf += Stringify(xpos);
-         strbuf += _(" ");
+         strbuf += wxT(" ");
          strbuf += Stringify(ypos);
       }
       DisplayText(dc, strbuf, h_xy, BASELINE1);
@@ -542,7 +548,7 @@ StatusBar::StatusBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int h
       int major, minor;
       wxGetOsVersion(&major, &minor);
       if ( major > 5 || (major == 5 && minor >= 1) ) {
-         // 5.1+ means XP or later (Vista if major >= 6)
+         // 5.1+ means XP or later (Vista or later if major >= 6)
          textascent = 11;
       } else {
          textascent = 10;
@@ -551,7 +557,12 @@ StatusBar::StatusBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int h
       // use smaller font on GTK
       statusfont = wxFont::New(8, wxMODERN, wxNORMAL, wxNORMAL);
       textascent = 11;
+   #elif defined(__WXOSX_COCOA__)
+      // we need to specify facename to get Monaco instead of Courier
+      statusfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Monaco"));
+      textascent = 10;
    #else
+      // __WXMAC__
       statusfont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL);
       textascent = 10;
    #endif
