@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -152,11 +152,19 @@ static int mindelpos;                  // minimum x position of DELETE_BUTT
 const int TBARHT = 32;                 // height of timeline bar
 const int SCROLLHT = 17;               // height of scroll bar
 const int PAGESIZE = 10;               // scroll amount when paging
-const int BUTTON_WD = 24;              // nominal width of bitmap buttons
 
 const int MINSPEED = -10;              // minimum autoplay speed
 const int MAXSPEED = 10;               // maximum autoplay speed
 
+// width and height of bitmap buttons
+#if defined(__WXOSX_COCOA__) || defined(__WXGTK__)
+   const int BUTTON_WD = 28;
+   const int BUTTON_HT = 28;
+#else
+   const int BUTTON_WD = 24;
+   const int BUTTON_HT = 24;
+#endif
+
 // timeline bar buttons (must be global to use Connect/Disconnect on Windows)
 static wxBitmapButton* tlbutt[NUM_BUTTONS];
 
@@ -197,7 +205,7 @@ TimelineBar::TimelineBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, i
       smallgap = 6;
    #else
       xpos = 4;
-      ypos = 4;
+      ypos = (32 - BUTTON_HT) / 2;
       smallgap = 4;
    #endif
    biggap = 16;
@@ -224,6 +232,10 @@ TimelineBar::TimelineBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, i
       // use smaller font on GTK
       timelinefont = wxFont::New(8, wxMODERN, wxNORMAL, wxNORMAL);
       textascent = 11;
+   #elif defined(__WXOSX_COCOA__)
+      // we need to specify facename to get Monaco instead of Courier
+      timelinefont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Monaco"));
+      textascent = 10;
    #elif defined(__WXMAC__)
       timelinefont = wxFont::New(10, wxMODERN, wxNORMAL, wxNORMAL);
       textascent = 10;
@@ -445,6 +457,9 @@ void TimelineBar::OnButton(wxCommandEvent& event)
    // call MainFrame::OnMenu after OnButton finishes
    wxCommandEvent cmdevt(wxEVT_COMMAND_MENU_SELECTED, cmdid);
    wxPostEvent(mainptr->GetEventHandler(), cmdevt);
+
+   // avoid possible problems
+   viewptr->SetFocus();
 }
 
 // -----------------------------------------------------------------------------
@@ -589,7 +604,7 @@ void TimelineBar::OnButtonUp(wxMouseEvent& event)
       // call OnButton
       wxCommandEvent buttevt(wxEVT_COMMAND_BUTTON_CLICKED, id);
       buttevt.SetEventObject(tlbutt[id]);
-      tlbutt[id]->ProcessEvent(buttevt);
+      tlbutt[id]->GetEventHandler()->ProcessEvent(buttevt);
    }
 }
 
@@ -597,7 +612,8 @@ void TimelineBar::OnButtonUp(wxMouseEvent& event)
 
 void TimelineBar::AddButton(int id, const wxString& tip)
 {
-   tlbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos));
+   tlbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos),
+                                   wxSize(BUTTON_WD, BUTTON_HT));
    if (tlbutt[id] == NULL) {
       Fatal(_("Failed to create timeline bar button!"));
    } else {
@@ -814,10 +830,10 @@ void StartStopRecording()
 {
    if (!inscript && currlayer->algo->hyperCapable()) {
       if (currlayer->algo->isrecording()) {
-         // terminate GeneratePattern()
+         // terminate GeneratePattern
          mainptr->Stop();
       } else {
-         if (currlayer->algo->isEmpty()) {   
+         if (currlayer->algo->isEmpty()) {
             statusptr->ErrorMessage(_("There is no pattern to record."));
             return;
          }
@@ -846,19 +862,7 @@ void StartStopRecording()
             mainptr->GeneratePattern();
             allowundo = saveundo;
             
-            // recording has stopped
-            currlayer->algo->stoprecording();
-            if (currlayer->algo->getframecount() > 0) {
-               // probably best to go to last frame
-               currlayer->currframe = currlayer->algo->getframecount() - 1;
-               currlayer->autoplay = 0;
-               currlayer->tlspeed = 0;
-               currlayer->algo->gotoframe(currlayer->currframe);
-               if (currlayer->autofit) viewptr->FitInView(1);
-            }
-
-            if (!showtimeline) ToggleTimelineBar();
-            mainptr->UpdateUserInterface(true);
+            // GeneratePattern has called currlayer->algo->stoprecording()
          } else {
             // can this ever happen???!!!
             Warning(_("Could not start recording!"));
