@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2011 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -358,10 +358,10 @@ bool ChangeNode::DoChange(bool undo)
 
       case rulechange:
          if (undo) {
-            currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+            RestoreRule(oldrule);
             currlayer->currsel = oldsel;
          } else {
-            currlayer->algo->setrule( newrule.mb_str(wxConvLocal) );
+            RestoreRule(newrule);
             currlayer->currsel = newsel;
          }
          // show new rule in window title (file name doesn't change)
@@ -657,7 +657,8 @@ void UndoRedo::SaveCurrentPattern(const wxString& tempfile)
    //!!! need lifealgo::CanWriteFormat(MC_format) method???
    if ( currlayer->algo->hyperCapable() ) {
       // save hlife pattern in a macrocell file
-      err = mainptr->WritePattern(tempfile, MC_format, 0, 0, 0, 0);
+      err = mainptr->WritePattern(tempfile, MC_format, no_compression,
+                                  0, 0, 0, 0);
    } else {
       // can only save RLE file if edges are within getcell/setcell limits
       bigint top, left, bottom, right;
@@ -667,7 +668,7 @@ void UndoRedo::SaveCurrentPattern(const wxString& tempfile)
       } else {
          // use XRLE format so the pattern's top left location and the current
          // generation count are stored in the file
-         err = mainptr->WritePattern(tempfile, XRLE_format,
+         err = mainptr->WritePattern(tempfile, XRLE_format, no_compression,
                                      top.toint(), left.toint(),
                                      bottom.toint(), right.toint());
       }
@@ -1265,7 +1266,7 @@ void UndoRedo::UndoChange()
       // terminate generating loop and set command_pending flag
       mainptr->Stop();
       mainptr->command_pending = true;
-      mainptr->cmdevent.SetId(wxID_UNDO);
+      mainptr->cmdevent.SetId(ID_UNDO);
       return;
    }
    
@@ -1331,7 +1332,7 @@ void UndoRedo::RedoChange()
       // terminate generating loop and set command_pending flag
       mainptr->Stop();
       mainptr->command_pending = true;
-      mainptr->cmdevent.SetId(wxID_REDO);
+      mainptr->cmdevent.SetId(ID_REDO);
       return;
    }
    */
@@ -1429,7 +1430,7 @@ void UndoRedo::UpdateUndoItem(const wxString& action)
       wxString label = _("Undo ");
       label += action;
       label += GetAccelerator(DO_UNDO);
-      mbar->SetLabel(wxID_UNDO, label);
+      mbar->SetLabel(ID_UNDO, label);
    }
 }
 
@@ -1444,7 +1445,7 @@ void UndoRedo::UpdateRedoItem(const wxString& action)
       wxString label = _("Redo ");
       label += action;
       label += GetAccelerator(DO_REDO);
-      mbar->SetLabel(wxID_REDO, label);
+      mbar->SetLabel(ID_REDO, label);
    }
 }
 
